package com.ruoyi.basic.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.nacos.shaded.com.google.gson.Gson;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ruoyi.basic.domain.LayoutConfig;

import com.ruoyi.basic.domain.LayoutDesc;
import com.ruoyi.basic.mapper.LayoutConfigDao;
import com.ruoyi.basic.mapper.LayoutDescDao;
import com.ruoyi.basic.service.LayoutConfigService;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.system.api.model.LoginUser;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;

/**
 * 布局配置表(LayoutConfig)表服务实现类
 *
 * @author makejava
 * @since 2024-08-26 16:38:47
 */
@Service("layoutConfigService")
public class LayoutConfigServiceImpl implements LayoutConfigService {
    @Resource
    private LayoutConfigDao layoutConfigDao;
    @Resource
    private LayoutDescDao layoutDescDao;

    /**
     * 通过ID查询单条数据
     *
     * @param configId 主键
     * @return 实例对象
     */
    @Override
    public LayoutConfig queryById(Long configId) {
        return this.layoutConfigDao.queryById(configId);
    }


    /**
     * 分页查询
     *
     * @param layoutConfig 筛选条件
     * @param pageRequest      分页对象
     * @return 查询结果
     */
    @Override
    public Page<LayoutConfig> queryByPage(LayoutConfig layoutConfig, PageRequest pageRequest) {
        long total = this.layoutConfigDao.count(layoutConfig);
        return new PageImpl<>(this.layoutConfigDao.queryAllByLimit(layoutConfig, pageRequest), pageRequest, total);
    }

    /**
     * 新增数据
     *
     * @param layoutConfig 实例对象
     * @return 实例对象
     */
    @Override
    public LayoutConfig insert(LayoutConfig layoutConfig) {
        this.layoutConfigDao.insert(layoutConfig);
        return layoutConfig;
    }

    /**
     * 修改数据
     *
     * @param layoutConfig 实例对象
     * @return 实例对象
     */
    @Override
    public LayoutConfig update(LayoutConfig layoutConfig) {
        this.layoutConfigDao.update(layoutConfig);
        return this.queryById(layoutConfig.getConfigId());
    }

    /**
     * 通过主键删除数据
     *
     * @param configId 主键
     * @return 是否成功
     */
    @Override
    public boolean deleteById(Long configId) {
        return this.layoutConfigDao.deleteById(configId) > 0;
    }

    @Override
    public AjaxResult insertSceneConfig(List<Object> items, String option) throws JsonProcessingException {
        String layoutId = UUID.randomUUID().toString().replaceAll("-", "");
        LoginUser loginUser = SecurityUtils.getLoginUser();
        LayoutDesc layoutDesc = new LayoutDesc();
        layoutDesc.setLayoutId(layoutId);
        layoutDesc.setSceneConfig(option);
//        layoutDesc.setCreateBy(loginUser.getUsername());
        Date date = new Date();
        layoutDesc.setCreateTime(date);
        List<LayoutConfig> layoutConfigs = new ArrayList<>();
        for (Object item : items) {
            LayoutConfig layoutConfig = new LayoutConfig();
            layoutConfig.setLayoutId(layoutId);
            ObjectMapper objectMapper = new ObjectMapper();
            String itemString = objectMapper.writeValueAsString(item);
            JsonNode jsonNode = objectMapper.readTree(itemString);
            String type = jsonNode.get("type").asText();
            layoutConfig.setType(type);
            layoutConfig.setConfigDesc(itemString);
//            layoutConfig.setCreateBy(loginUser.getUsername());
            layoutConfig.setCreateTime(date);
            layoutConfigs.add(layoutConfig);
        }
        int descCount = layoutDescDao.insert(layoutDesc);
        int configCount = layoutConfigDao.insertBatch(layoutConfigs);
        return AjaxResult.success();
    }

    @Override
    public AjaxResult selectConfigById(String sceneId) {
        List<LayoutConfig> layoutConfigs = layoutConfigDao.selectConfigById(sceneId);
        return AjaxResult.success(layoutConfigs);
    }

    @Override
    public AjaxResult selectAllScenes() {
        List<LayoutDesc> layoutDescs = layoutDescDao.selectAllScenes();
        return AjaxResult.success(layoutDescs);
    }
}
