package com.ruoyi.business.controller;

import com.ruoyi.business.domain.*;
import com.ruoyi.business.domain.VO.AllNumsVo;
import com.ruoyi.business.service.*;
import com.ruoyi.common.core.constant.HwDictConstants;
import com.ruoyi.common.core.web.controller.BaseController;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.common.core.web.page.TableDataInfo;
import com.ruoyi.common.log.annotation.Log;
import com.ruoyi.common.log.enums.BusinessType;
import com.ruoyi.common.security.annotation.RequiresPermissions;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.system.api.domain.SysUser;
import com.ruoyi.system.api.model.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Description: 移动端Controller
 * @ClassName: HwAppController
 * @Date    :2023-08-31  17:46
 * @Version :1.0
 */
@RestController
@RequestMapping("/app")
public class HwAppController extends BaseController {

    @Autowired
    private IHwMonitorUnitService hwMonitorUnitService;

    @Autowired
    private IHwSceneService hwSceneService;

    @Autowired
    private IHwAlarmInfoService hwAlarmInfoService;

    @Autowired
    private IHwDeviceService hwDeviceService;

    @Autowired
    private IHwDeviceModeService hwDeviceModeService;

    /**
     * 获取场景，如果是管理员，则可以获取所有场景
     */
    @RequiresPermissions("business:app:index")
    @GetMapping("/selectSecnes")
    public AjaxResult selectSecnes(HwScene scene) {
        List<HwScene> hwScenes = hwSceneService.selectHwSceneList(scene);
        return success(hwScenes);
    }

    /**
     * 	按场景展示总的设备数量和总的最子级监控单元数量（app首页上方）
     */
    @RequiresPermissions("business:app:index")
    @GetMapping("/sceneAllNums/{sceneId}")
    public AllNumsVo sceneAllNums(@PathVariable("sceneId") Long sceneId) {
        return hwMonitorUnitService.selectAllNums(sceneId);
    }

    /**
     * 分页获取最子集监控单元列表(首页)
     */
    @RequiresPermissions("business:app:index")
    @GetMapping("/limitSubMonitorUnit/{sceneId}")
    public TableDataInfo limitSubMonitorUnit(@PathVariable("sceneId") Long sceneId) {
        HwMonitorUnit hwMonitorUnit = new HwMonitorUnit();
        hwMonitorUnit.setSceneId(sceneId);
        startPage();
        List<HwMonitorUnit> hwMonitorUnits = hwMonitorUnitService.selectLimitSubMonitorUnit(hwMonitorUnit);
        return getDataTable(hwMonitorUnits);
    }

    /**
     * @return TableDataInfo
     * @param: hwAlarmInfo
     * @description 获取报警信息（手机首页）
     * @author xins
     * @date 2023-09-15 11:03
     */
    @GetMapping("/getAlarmInfos")
    @RequiresPermissions("business:app:index")
    public TableDataInfo getAlarmInfos(HwAlarmInfo hwAlarmInfo) {
        hwAlarmInfo.setHandleStatus(HwDictConstants.ALARM_HANDLE_STATUS_NO);
        List<HwAlarmInfo> alarmInfos = hwAlarmInfoService.selectHwAlarmInfoList(hwAlarmInfo);
        return getDataTable(alarmInfos);
    }

    /**
     * @param: hwAlarmInfo
     * @description 处理报警信息
     * @author xins
     * @date 2023-09-28 9:59
     * @return AjaxResult
     */
    @PutMapping("/handleAlarmInfo")
    @RequiresPermissions("business:app:index")
    public AjaxResult handleAlarmInfo(@RequestBody HwAlarmInfo hwAlarmInfo) {
        hwAlarmInfo.setUpdateBy(SecurityUtils.getUsername());
        return toAjax(hwAlarmInfoService.updateHwAlarmInfo(hwAlarmInfo));
    }


    /**
     * 树状获取监控单元列表(手机端监控单元页面)
     *
     * @param sceneId 场景ID
     * @return list
     * @throws
     */
    @RequiresPermissions("business:app:monitorUnit")
    @GetMapping("/treeList/{sceneId}")
    public AjaxResult monitorUnitTree(@PathVariable("sceneId") Long sceneId) {
        HwMonitorUnit queryMonitorUnit = new HwMonitorUnit();
        queryMonitorUnit.setSceneId(sceneId);
        return success(hwMonitorUnitService.selectMonitorTreeList(queryMonitorUnit));
    }

    /**
     * @param: monitorUnitId
     * @description 根据监控单元ID获取在线和离线设备数量，正常和告警设备数量
     * @author xins
     * @date 2023-09-28 13:31
     * @return AjaxResult
     */
//    @RequiresPermissions("business:app:monitorUnit")
    @GetMapping("/getDevicesInfoByMonitorUnitId/{monitorUnitId}")
    public AjaxResult getDevicesInfoByMonitorUnitId(@PathVariable("monitorUnitId") Long monitorUnitId){
//        HwDevice queryDevice = new HwDevice();
//        queryDevice.setMonitorUnitId(monitorUnitId);
        return success(hwDeviceService.getDevicesInfoByMonitorUnitId(monitorUnitId));
    }

    /**
     * 根据监控单元ID获取设备信息，分可控制设备和上传数据设备（手机监控单元页面）
     *
     * @param monitorUnitId 监控单元id
     */
    @RequiresPermissions("business:app:monitorUnit")
    @GetMapping("/selectDevicesByMonitorUnitId/{monitorUnitId}/{sceneId}")
    public AjaxResult selectDeviceByDeviceModeByMonitorUnitId(@PathVariable("monitorUnitId") Long monitorUnitId,
                                                              @PathVariable("sceneId") Long sceneId) {
        HwDevice queryHwDevice = new HwDevice();
        queryHwDevice.setMonitorUnitId(monitorUnitId);
        queryHwDevice.setSceneId(sceneId);

        return success(hwDeviceService.getDevicesByMonitor(queryHwDevice));
    }


    /**
     * 新增设备信息
     */
    @RequiresPermissions("business:app:deviceAdd")
    @Log(title = "设备信息（手机端）", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody HwDevice hwDevice) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        SysUser user = loginUser.getSysUser();
        hwDevice.setTenantId(user.getTenantId());
        hwDevice.setCreateBy(user.getUserName());
        return toAjax(hwDeviceService.insertHwDevice(hwDevice));
    }

    /**
     * 根据监设备编号获取设备信息，（手机扫一扫）
     *
     * @param deviceCode 设备编号
     */
    @RequiresPermissions("business:app:device")
    @GetMapping("/getDeviceByDeviceCode/{deviceCode}")
    public AjaxResult getDeviceByDeviceCode(@PathVariable("deviceCode") String deviceCode){
        return success(hwDeviceService.selectHwDeviceByDeviceCode(deviceCode));
    }

    /**
     * 根据场景获取设备模型列表
     *
     * @param sceneId 场景ID
     */
    @RequiresPermissions("business:app:deviceAdd")
    @GetMapping("/getDeviceModesBySceneId/{sceneId}")
    public AjaxResult getDeviceModesBySceneId(@PathVariable("sceneId") Long sceneId){
        HwDeviceMode queryDeviceMode = new HwDeviceMode();
        queryDeviceMode.setSceneId(sceneId);
        queryDeviceMode.setDeviceModeStatus(HwDictConstants.DEVICE_MODE_STATUS_NORMAL);
        return success(hwDeviceModeService.selectHwDeviceModeList(queryDeviceMode));
    }

    /**
     * 根据场景获取网关设备列表
     *
     * @param sceneId 场景ID
     */
    @RequiresPermissions("business:app:deviceAdd")
    @GetMapping("/getGatewayDevicesBySceneId/{sceneId}")
    public AjaxResult getGatewayDevicesBySceneId(@PathVariable("sceneId") Long sceneId){
        HwDevice queryDevice = new HwDevice();
        queryDevice.setSceneId(sceneId);
        queryDevice.setDeviceType(HwDictConstants.DEVICE_TYPE_GATEWAY_DEVICE);
        return success(hwDeviceService.selectHwDeviceList(queryDevice));
    }

}
