package com.ruoyi.business.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.parser.Feature;
import com.ruoyi.business.domain.*;
import com.ruoyi.business.domain.VO.*;
import com.ruoyi.business.service.*;
import com.ruoyi.business.utils.ExcelUtils;
import com.ruoyi.common.core.constant.HwDictConstants;
import com.ruoyi.common.core.utils.poi.ExcelUtil;
import com.ruoyi.common.core.web.controller.BaseController;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.common.core.web.page.TableDataInfo;
import com.ruoyi.common.core.web.page.TableDataNameVo;
import com.ruoyi.common.log.annotation.Log;
import com.ruoyi.common.log.enums.BusinessType;
import com.ruoyi.common.security.annotation.RequiresPermissions;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.tdengine.api.domain.TdReturnDataVo;
import io.micrometer.core.instrument.util.StringEscapeUtils;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.data.redis.RedisProperties;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/monitorPlatform")
public class HwMonitorPlatformController extends BaseController {

    @Autowired
    private IHwMonitorUnitService hwMonitorUnitService;
    @Autowired
    private IHwDeviceService hwDeviceService;
    @Autowired
    private IHwDeviceModeService hwDeviceModeService;
    @Autowired
    private IHwSceneService hwSceneService;
    @Autowired
    private IHwElectronicFenceService hwElectronicFenceService;
    @Autowired
    private IHwAlarmInfoService hwAlarmInfoService;

    @Autowired
    private IHwMonitorPlatformService hwMonitorPlatformService;

    /**
     * 查询3.	设备各类型占所有设备数量百分比饼状图（首页左下角）
     */
    @RequiresPermissions("business:monitor:index")
    @GetMapping("/monitorUnitPercentage")
    public TableDataInfo monitorUnitPercentage() {
        List<DeviceModeVo> maps = hwDeviceService.selectDeviceModeNameVo();
        return getDataTable(maps);
    }

    /**
     * 各场景监控单元数量占监控单元总数量百分比图(首页左上)
     */
    @RequiresPermissions("business:monitor:index")
    @GetMapping("/monitorPercentage")
    public TableDataInfo monitorPercentage() {
        List<HwMonitorUnitVo> maps = hwMonitorUnitService.selectMonitorPercentage();
        return getDataTable(maps);
    }

    /**
     * 	展示总的设备数量和总的监控单元数量（首页中间上方）
     */
    @RequiresPermissions("business:monitor:index")
    @GetMapping("/allNums")
    public AllNumsVo allNums() {
        AllNumsVo allNumsVos = hwMonitorUnitService.selectAllNums(null);

        return allNumsVos;
    }

    /**
     * @return TableDataInfo
     * @param: hwAlarmInfo
     * @description 获取报警信息（首页右下角，定位场景首页，标准场景首页）
     * @author xins
     * @date 2023-09-15 11:03
     */
    @GetMapping("/getAlarmInfos")
    @RequiresPermissions("business:monitor:alarm")
    public TableDataInfo getAlarmInfos(HwAlarmInfo hwAlarmInfo) {
        //startPage();
        hwAlarmInfo.setHandleStatus(HwDictConstants.ALARM_HANDLE_STATUS_NO);
        List<HwAlarmInfo> alarmInfos = hwAlarmInfoService.selectHwAlarmInfoList(hwAlarmInfo);
        return getDataTable(alarmInfos);
    }

    @PutMapping("/handleAlarmInfo")
    @Log(title = "监控平台处理报警信息", businessType = BusinessType.UPDATE)
    @RequiresPermissions("business:monitor:alarm")
    public AjaxResult handleAlarmInfo(@RequestBody HwAlarmInfo hwAlarmInfo) {
        hwAlarmInfo.setUpdateBy(SecurityUtils.getUsername());
        return toAjax(hwAlarmInfoService.updateHwAlarmInfo(hwAlarmInfo));
    }

    /**
     * 2.	设备实时在运数量曲线图(首页右上角)
     *
     * @param sceneId 场景id
     */
    @RequiresPermissions("business:monitor:index")
    @GetMapping("/getDeviceOperations/{sceneId}")
    public AjaxResult selectDeviceStatus(@PathVariable("sceneId") Long sceneId) {
//        System.out.println(("sceneid:" + sceneId));
        return AjaxResult.success(hwDeviceService.getOnlineDevicesCount(sceneId));
    }


    /**
     * 	按场景展示总的设备数量和总的监控单元数量（标准场景）
     */
    @RequiresPermissions("business:monitor:standard")
    @GetMapping("/sceneAllNums/{sceneId}")
    public AllNumsVo sceneAllNums(@PathVariable("sceneId") Long sceneId) {
        AllNumsVo allNumsVos = hwMonitorUnitService.selectAllNums(sceneId);
        return allNumsVos;
    }

    /**
     * 分页获取最子集监控单元列表(标准场景右侧)
     */
    @RequiresPermissions("business:monitor:standard")
    @GetMapping("/limitSubMonitorUnit/{sceneId}")
    public TableDataInfo limitSubMonitorUnit(@PathVariable("sceneId") Long sceneId) {
        HwMonitorUnit hwMonitorUnit = new HwMonitorUnit();
        hwMonitorUnit.setSceneId(sceneId);
        startPage();
        List<HwMonitorUnit> hwMonitorUnits = hwMonitorUnitService.selectLimitSubMonitorUnit(hwMonitorUnit);
        return getDataTable(hwMonitorUnits);
    }

    /**
     * @return AjaxResult
     * @param: sceneId
     * @description 标准场景按报警类型返回其报警数量
     * @author xins
     * @date 2023-09-21 11:35
     */
    @RequiresPermissions("business:monitor:standard")
    @GetMapping("/getAlarmCountGroupAlarmType/{sceneId}")
    public AjaxResult getAlarmCountGroupAlarmType(@PathVariable("sceneId") Long sceneId) {
        return success(hwAlarmInfoService.selectAlarmsByAlarmType(sceneId));
    }

    /**
     * 按电子围栏定位显示其下设备数量，以及报警的设备数量(定位场景地图气泡图)
     *
     * @param sceneId
     */
    @RequiresPermissions("business:monitor:gps")
    @GetMapping("/ElectronicNumVo/{sceneId}")
    public AjaxResult ElectronicNumVo(@PathVariable("sceneId") Long sceneId) {

        return success(hwElectronicFenceService.selectEleTypeNums(sceneId));
    }


    /**
     * 监控单元报警统计
     */
    @GetMapping("/alarmStats/{sceneId}")
    public AjaxResult alarmStats(@PathVariable("sceneId") Long sceneId) {
        List<AlarmStatsVo> alarmStatsVos = hwMonitorUnitService.selectAlarmStats(sceneId);
        return success(alarmStatsVos);
    }

    /**
     * 获取最子级监控单元数量、设备数量和在线设备数量（定位场景）
     *
     * @param sceneId 场景id
     */
    @RequiresPermissions("business:monitor:gps")
    @GetMapping("/subDevice/{sceneId}")
    public SubDeviceSumVo subDevice(@PathVariable("sceneId") Long sceneId) {
        return hwMonitorUnitService.selectSubDeviceSum(sceneId);
    }


    /**
     * 树状获取监控单元列表(设备监测左侧树)
     *
     * @param sceneId 场景ID
     * @return list
     * @throws
     */
    @RequiresPermissions("business:monitor:deviceMonitor")
    @GetMapping("/treeList/{sceneId}")
    public AjaxResult monitorUnitTree(@PathVariable("sceneId") Long sceneId) {
        HwMonitorUnit queryMonitorUnit = new HwMonitorUnit();
        queryMonitorUnit.setSceneId(sceneId);
        return success(hwMonitorUnitService.selectMonitorTreeList(queryMonitorUnit));
    }

    /**
     * 获取监控单元详细信息
     */
//    @RequiresPermissions("business:monitorUnit:query")
    @RequiresPermissions("business:monitor:deviceMonitor")
    @GetMapping(value = "/getMonitorById/{monitorUnitId}")
    public AjaxResult getMonitorById(@PathVariable("monitorUnitId") Long monitorUnitId) {
        return success(hwMonitorUnitService.selectHwMonitorUnitJoinByMonitorUnitId(monitorUnitId));
    }

    /**
     * 根据监控单元ID获取设备信息，分可控制设备和上传数据设备（设备监测页面）
     *
     * @param monitorUnitId 监控单元id
     */
    @RequiresPermissions("business:monitor:deviceMonitor")
    @GetMapping("/selectDevicesByMonitorUnitId/{monitorUnitId}/{sceneId}")
    public AjaxResult selectDeviceByDeviceModeByMonitorUnitId(@PathVariable("monitorUnitId") Long monitorUnitId,
                                                              @PathVariable("sceneId") Long sceneId) {
        HwDevice queryHwDevice = new HwDevice();
        queryHwDevice.setMonitorUnitId(monitorUnitId);
        queryHwDevice.setSceneId(sceneId);

        return success(hwDeviceService.getDevicesByMonitor(queryHwDevice));
    }


    @RequiresPermissions("business:monitor:deviceMonitor")
    @Log(title = "监控平台下发控制指令", businessType = BusinessType.CONTROL)
    @PostMapping("/publishControlCommand")
    public AjaxResult publishControlCommand(@RequestBody DeviceControlVo deviceControlVo) {
        hwDeviceService.publishControlCommand(deviceControlVo);
        return success();
    }


    /**
     * @description 根据场景ID获取设备模型列表(设备汇总页面)
     * @author xins
     * @date 2023-09-16 12:44
     */
    @RequiresPermissions("business:monitor:deviceList")
    @PostMapping("/getDeviceModesBySceneId")
    public AjaxResult getDeviceModesBySceneId(@RequestBody HwDeviceMode queryHwDeviceMode) {
        return success(hwDeviceModeService.selectHwDeviceModeList(queryHwDeviceMode));
    }

    /**
     * @return AjaxResult
     * @param: monitorUnitId
     * @description 根据设备模型获取最新数据（设备汇总页面）
     * @author xins
     * @date 2023-09-16 12:45
     */
    @RequiresPermissions("business:monitor:deviceList")
    @PostMapping("/selectLatestDataByDeviceMode")
    public AjaxResult selectLatestDataByDeviceMode(@Validated @RequestBody DeviceLatestDataVo deviceLatestData) {
        JSONObject returnObj = hwMonitorPlatformService.selectLatestDataByTags(deviceLatestData);
        return success(returnObj);
    }

    @RequiresPermissions("business:monitor:deviceInfo")
    @GetMapping("/getDeviceInfo/{deviceId}")
    public AjaxResult getDeviceInfo(@PathVariable("deviceId") Long deviceId) {
        HwDevice device = hwDeviceService.selectHwDeviceJoinByDeviceId(deviceId);
        return success(device);
    }

    @RequiresPermissions("business:monitor:deviceInfo")
    @GetMapping("/getLinkDevices/{deviceId}")
    public AjaxResult getLinkDevices(@PathVariable("deviceId") Long deviceId) {
        return success(hwDeviceService.selectLinkDeviceList(deviceId));
    }

    /**
     * @return AjaxResult
     * @param: deviceHistoryData
     * @description 获取设备历史数据
     * @author xins
     * @date 2023-09-16 15:15
     */
    @RequiresPermissions("business:monitor:deviceInfo")
    @PostMapping("/getHistoryData")
    public AjaxResult getHistoryData(@Validated @RequestBody DeviceHistoryDataVo deviceHistoryData) {
        JSONObject returnObj = new JSONObject();
        TdReturnDataVo returnDataVo = hwMonitorPlatformService.getHistoryData(deviceHistoryData);
        if (returnDataVo != null) {
            List<DeviceDataColumnVo> deviceDataColumnVos = hwMonitorPlatformService.
                    getDeviceDataColumns(deviceHistoryData.getDeviceModeId());
            returnObj.put("deviceDataColumns", deviceDataColumnVos);
            returnObj.put("historyData", returnDataVo);
        }

        return success(returnObj);

    }


    /**
     * 获取场景，如果是管理员，则可以获取所有场景
     */
    @GetMapping("/selectSecnes")
    public AjaxResult selectSecnes(HwScene scene) {
        List<HwScene> hwScenes = hwSceneService.selectHwSceneList(scene);
        return success(hwScenes);
    }

    /**
     * 根据场景查询场景下设备的经纬度
     * */
    @GetMapping("/selectDeviceLatitudeAndLongitude/{sceneId}")
    public AjaxResult selectDeviceLatitudeAndLongitude(@PathVariable("sceneId") Long sceneId){
        List<BeaconDevice> beaconDevices = hwDeviceService.selectDeviceLatitudeAndLongitude(sceneId);
        return success(beaconDevices);
    }
    /**
     * 根据设备id查询信标设备历史数据
     * */
    @GetMapping("/selectBeaconDevicesHistory")
    public TableDataInfo selectBeaconDevicesHistory(@RequestParam(required = false) Map map){
//        List<BeaconDevice> beaconDevices = hwDeviceService.selectBeaconDevicesHistory(map);
        List<Map<String, Object>> lists = hwDeviceService.selectBeaconDevicesHistory(map);
        if(lists != null){
            long total = Long.valueOf(lists.get(lists.size() - 1).get("total").toString());
            lists.remove(lists.size() - 1);
            TableDataInfo dataTable = getDataTable(lists);
            dataTable.setTotal(total);
            return dataTable;
        }else {
            return null;
        }
    }
    /**
     * 根据设备id查询其对应的监控单元
     * */
    @GetMapping("/selectMonitorUnit/{deviceId}")
    public AjaxResult selectMonitorUnit(@PathVariable("deviceId") Long deviceId){
        HwMonitorUnit hwMonitorUnit = hwDeviceService.selectMonitorUnit(deviceId);
        return success(hwMonitorUnit);
    }
    /**
     * 根据设备id查询其对应的电子围栏
     * */
    @GetMapping("/selectMonitorElectronic/{deviceId}")
    public AjaxResult selectMonitorElectronic(@PathVariable("deviceId") Long deviceId){
        return success(hwDeviceService.selectMonitorElectronic(deviceId));
    }
    /**
     * 历史数据弹出框（多参数且可多选）
     * */
    @GetMapping("/selectHistoryAndTrendAnalysis")
    public TableDataNameVo selectHistoryDevice(@RequestParam(required = false) Map map){
        List<Map<String, Object>> list = hwDeviceService.selectHistoryDevice(map);
        TableDataInfo dataTable = getDataTable(list);
        TableDataNameVo tableDataNameVo = new TableDataNameVo();
        if (list.size()>1){
            long total = Long.valueOf(list.get(list.size() - 1).get("total").toString());
            Map<String, Object> mapName = new HashMap<>();
            list.remove(list.size() - 1);
            if (list.size()>0){
                mapName = list.get(list.size() - 1);
                list.remove(list.size() - 1);
            }


            Map<String, Object> mapTotal = new HashMap<String, Object>();
            dataTable = getDataTable(list);
            dataTable.setTotal(total);
            tableDataNameVo.setTableDataInfo(dataTable);
            tableDataNameVo.setMapName(mapName);
            return tableDataNameVo;
        }
        else {
            map.remove("startTime");
            map.remove("endTime");
            List<Map<String, Object>> listNull = hwDeviceService.selectHistoryDevice(map);
            Map<String, Object> map1 = new HashMap<String, Object>();
            if (listNull.size()>1){
                 map1 = listNull.get(listNull.size() - 2);
            }
            dataTable.setTotal(0);
            list.get(0).remove("total");
            tableDataNameVo.setMapName(map1);
            tableDataNameVo.setTableDataInfo(dataTable);
            return tableDataNameVo;

        }

    }
    /**
     * 返回一级、二级、传感器
     * */
    @GetMapping("/selectComparison")
    public AjaxResult selectComparison(){
        List<TreeDeviceVo> list = hwDeviceService.selectComparison();
        return success(list);
    }
    /**
     * 弹出框导出数据
     * */
    @PostMapping("/export")
    public void export(HttpServletResponse response,@RequestParam Map map)
    {
        List<Map<String, Object>> list = hwDeviceService.getExportDevice(map);
        List<DeviceExport> exportList = new ArrayList<>();
        for (int i = 0; i < list.size(); i++){
            DeviceExport deviceExport = new DeviceExport();
            if (list.get(i).containsKey("deviceId")){
                deviceExport.setDeviceId(Long.valueOf(list.get(i).get("deviceId").toString()));
            }
            if (list.get(i).containsKey("value1")){
                deviceExport.setValue1(list.get(i).get("value1").toString());
            }
            if (list.get(i).containsKey("voltage")){
                deviceExport.setVoltage(list.get(i).get("voltage").toString());
            }
            if (list.get(i).containsKey("longitude")){
                deviceExport.setLongitude(list.get(i).get("longitude").toString());
            }
            if (list.get(i).containsKey("latitude")){
                deviceExport.setLatitude(list.get(i).get("latitude").toString());
            }
            if (list.get(i).containsKey("ts")){
                deviceExport.setTs(list.get(i).get("ts").toString());
            }
            exportList.add(deviceExport);
        }
        ExcelUtil<DeviceExport> util = new ExcelUtil<DeviceExport>(DeviceExport.class);
        System.out.println(exportList.size());
        util.exportExcel(response,exportList,"历史数据导出");

    }

   /**
    * 进入实际地图后显示计量箱点位标识
    * */
   @GetMapping("/getDeviceByAreaId/{tenantId}")
    public AjaxResult getDeviceByAreaId(@PathVariable("tenantId") Long tenantId){
       List<HwMonitorUnit> deviceList = hwDeviceService.getDeviceByAreaId(tenantId);
       return success(deviceList);
   }
   /**
    * 报警信息导出
    * */
   @PostMapping("/AlarmInfosExport")
   @RequiresPermissions("business:monitor:alarm")
   public void AlarmInfosExport(HttpServletResponse response,Date startTime,Date endTime) {
       List<AlarmInfoExportVo> alarmInfoExportVos = hwAlarmInfoService.selectAlarmInfoExport(startTime,endTime);
       ExcelUtil<AlarmInfoExportVo> util = new ExcelUtil<AlarmInfoExportVo>(AlarmInfoExportVo.class);
       util.exportExcel(response,alarmInfoExportVos,"报警数据导出");
   }
    /**
     * 导入excel，同一个租户重复导入时会覆盖之前的excel
     * */
    @PostMapping("/import")
    public JSONArray importUser(@RequestPart("file")MultipartFile file,Long tenantId) throws Exception {
        JSONArray array = ExcelUtils.readMultipartFile(file);
//        System.out.println("导入数据为:" + array);
        hwDeviceService.insertRedis(array,tenantId);
        return array;
    }
    /**
     * 传入租户id查看导入的excel
     * */
    @GetMapping("/getRedisFile/{tenantId}")
    public JSONObject getRedisFile(@PathVariable("tenantId") Long tenantId)
    {
        JSONObject o = hwDeviceService.selectRedisFile(tenantId);

        if (!o.equals(null)&&!o.equals("")){

            return o;
        }
        else{
            return null;
        }
    }

    /**
     * 根据模糊查询返回信标设备信息
     * */
    @PostMapping("/selectDeviceByName")
    public AjaxResult selectDeviceByName(@RequestBody Map map){

        try {
            List<HwDevice> deviceList = hwDeviceService.selectMonitorUnitAndDeviceByName(map);
            return success(deviceList);
        }catch (Exception ex){
            return error(String.format("获取设备信息异常:%e",ex.getMessage()));
        }


    }


}
