package com.ruoyi.business.mapper;

import java.util.Date;
import java.util.List;
import com.ruoyi.business.domain.HwAlarmInfo;
import com.ruoyi.business.domain.VO.AlarmInfoExportVo;
import com.ruoyi.business.domain.VO.AlarmInfoVo;
import com.ruoyi.business.domain.VO.AlarmTypeVo;
import org.apache.ibatis.annotations.Param;

/**
 * 报警信息Mapper接口
 *
 * @author xins
 * @date 2023-09-15
 */
public interface HwAlarmInfoMapper
{
    /**
     * 查询报警信息
     *
     * @param alarmInfoId 报警信息主键
     * @return 报警信息
     */
    public HwAlarmInfo selectHwAlarmInfoByAlarmInfoId(Long alarmInfoId);

    /**
     * 查询报警信息列表
     *
     * @param hwAlarmInfo 报警信息
     * @return 报警信息集合
     */
    public List<HwAlarmInfo> selectHwAlarmInfoList(HwAlarmInfo hwAlarmInfo);

    /**
     * 新增报警信息
     *
     * @param hwAlarmInfo 报警信息
     * @return 结果
     */
    public int insertHwAlarmInfo(HwAlarmInfo hwAlarmInfo);

    /**
     * 修改报警信息
     *
     * @param hwAlarmInfo 报警信息
     * @return 结果
     */
    public int updateHwAlarmInfo(HwAlarmInfo hwAlarmInfo);


    /**
     * 删除报警信息
     *
     * @param alarmInfoId 报警信息主键
     * @return 结果
     */
    public int deleteHwAlarmInfoByAlarmInfoId(Long alarmInfoId);

    /**
     * 批量删除报警信息
     *
     * @param alarmInfoIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteHwAlarmInfoByAlarmInfoIds(Long[] alarmInfoIds);


    public List<AlarmTypeVo> selectAlarmsByAlarmType(Long sceneId);

    public AlarmInfoVo selectAlarmCountByFenceArea(Long fenceAreaId);

    /**
     * @param: monitorUnitId
     * @description 根据监控单元ID获取报警的设备数量
     * @author xins
     * @date 2023-09-28 14:17
     * @return int
     */
    public int selectAlarmCountByMonitorUnitId(Long monitorUnitId);

    /**
     * 导出报警信息
     * */
    List<AlarmInfoExportVo> selectAlarmInfoExport(@Param("startTime") Date startTime, @Param("endTime") Date endTime);

    /**
     * 批量处理报警信息
     * */
    public int updateHwAlarmInfoAllByDevice(HwAlarmInfo hwAlarmInfo);
}
