package com.ruoyi.business.mapper;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.ruoyi.business.domain.HwDevice;
import com.ruoyi.business.domain.VO.DeviceModeVo;
import com.ruoyi.business.domain.VO.HwMonitorUnitVo;
import com.ruoyi.business.domain.VO.TreeAreaVo;
import com.ruoyi.business.domain.VO.TreeDeviceVo;
import org.apache.ibatis.annotations.Param;

/**
 * 设备信息Mapper接口
 *
 * @author xins
 * @date 2023-08-24
 */
public interface HwDeviceMapper
{
    /**
     * 查询设备信息
     *
     * @param deviceId 设备信息主键
     * @return 设备信息
     */
    public HwDevice selectHwDeviceByDeviceId(Long deviceId);
    List<HwDevice> selectDeviceByMonitorId(Long monitorUnitId);
    Integer selectSumDeviceFlagNot0();

    /**
     * 查询3.	设备各类型占所有设备数量百分比饼状图（左下角）
     * */
    public List<DeviceModeVo> selectDeviceModeNameVo();
    /**
     * 查询设备信息列表
     *
     * @param hwDevice 设备信息
     * @return 设备信息集合
     */
    public List<HwDevice> selectHwDeviceList(HwDevice hwDevice);

    public List<HwDevice> selectDeviceByDeviceModeBySceneId(Long sceneId);
    public List<HwDevice> selectDeviceByDeviceModeByMonitorUnitId(Long monitor_unit_id);
    String selectDeviceByDeviceModeByDeviceModeFunction(Long deviceId);
    String selectDeviceModeFunctionByFunctionType(Long deviceId);
    List<HwDevice> selectDeviceNum(Long sceneId);

    /**
     * 新增设备信息
     *
     * @param hwDevice 设备信息
     * @return 结果
     */
    public int insertHwDevice(HwDevice hwDevice);

    /**
     * 修改设备信息
     *
     * @param hwDevice 设备信息
     * @return 结果
     */
    public int updateHwDevice(HwDevice hwDevice);

    /**
     * 删除设备信息(逻辑更新状态)
     *
     * @param deviceId 设备信息主键
     * @return 结果
     */
    public int deleteHwDeviceByDeviceId(Long deviceId);

    /**
     * 批量删除设备信息（物理删除）
     *
     * @param deviceIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteHwDeviceByDeviceIds(Long[] deviceIds);


    public int getOnlineDeviceNum(Long sceneId);


    public HwDevice selectHwDeviceJoinByDeviceId(Long deviceId);

    public List<HwDevice> selectLinkedDevices(Long deviceId);


    /**
     * 查询设备信息列表,join监控单元、设备模型等表
     *
     * @param hwDevice 设备信息
     * @return 设备信息集合
     */
    public List<HwDevice> selectHwDeviceJoinList(HwDevice hwDevice);


    /**
     * @param: releatedDeviceId
     * @description 校验网关设备是否有子设备
     * @author xins
     * @date 2023-09-27 15:29
     * @return int
     */
    public int checkExistSubDevice(Long releatedDeviceId);

    /**
     * @param: deviceCode
     * @description 根据设备编号获取设备信息
     * @author xins
     * @date 2023-09-28 13:23
     * @return HwDevice
     */
    public HwDevice selectHwDeviceByDeviceCode(String deviceCode);

    public int selectDeviceNumCount(HwDevice hwDevice);

    public int selectAbnormalDeviceNumCount(HwDevice hwDevice);


    public List<HwDevice> selectHwDeviceListByMonitor(HwDevice hwDevice);


    /**
     * @param: hwDevice
     * @description 获取离线设备报警规则已分配的设备
     * @author xins
     * @date 2023-11-14 9:13
     * @return List<HwDevice>
     */
    public List<HwDevice> selectAllocatedList(HwDevice hwDevice);

    /**
     * @param: hwDevice
     * @description 获取离线报警规则未分配的设备
     * @author xins
     * @date 2023-11-14 9:13
     * @return List<HwDevice>
     */
    public List<HwDevice>  selectUnallocatedList(HwDevice hwDevice);

    /**
     * 查询设备列表集合
     * */
    List<TreeDeviceVo> selectDeviceList();

    /**
     * 查询设备的字段名
     * */
    public String selectPropBydeviceIdAndFunctionName(@Param(value = "deviceModeId") Long deviceModeId,@Param(value = "functionName") String functionName);

    /**
     * 根据地图显示计量箱点位标识
     * */
    List<HwDevice> getDeviceByAreaId(@Param(value = "tenantId") Long tenantId,@Param(value = "areaId") Long areaId);
    List<TreeAreaVo> getTreeArea(Long areaId);

    /**
     * 获取监控单元下的信标设备
     * */
    List<HwDevice> getDeviceListInMonitorUnit(Long monitorUnitId);

    /**
     * 根据设备模型标识符获取设备模型名称
     * */
    String selectFunctionNameByFunctionIdentifier(String functionIdentifier);
    /**
     * 根据设备id查询该设备是否报警
     * */
    Long selectDeviceIfAlarm(Long deviceId);

    List<HwDevice> getDeviceByModel(Long modelId);
}
