package com.ruoyi.business.mapper;

import java.util.List;
import com.ruoyi.business.domain.HwDeviceMode;
import com.ruoyi.business.domain.HwDeviceModeFunction;
import com.ruoyi.business.domain.HwDeviceModeParameter;

/**
 * 设备模型Mapper接口
 *
 * @author xins
 * @date 2023-09-04
 */
public interface HwDeviceModeMapper
{
    /**
     * 查询设备模型
     *
     * @param deviceModeId 设备模型主键
     * @return 设备模型
     */
    public HwDeviceMode selectHwDeviceModeByDeviceModeId(Long deviceModeId);

    /**
     * 查询设备模型列表
     *
     * @param hwDeviceMode 设备模型
     * @return 设备模型集合
     */
    public List<HwDeviceMode> selectHwDeviceModeList(HwDeviceMode hwDeviceMode);

    /**
     * 新增设备模型
     *
     * @param hwDeviceMode 设备模型
     * @return 结果
     */
    public int insertHwDeviceMode(HwDeviceMode hwDeviceMode);

    /**
     * 修改设备模型
     *
     * @param hwDeviceMode 设备模型
     * @return 结果
     */
    public int updateHwDeviceMode(HwDeviceMode hwDeviceMode);

    /**
     * 删除设备模型
     *
     * @param deviceModeId 设备模型主键
     * @return 结果
     */
    public int deleteHwDeviceModeByDeviceModeId(Long deviceModeId);

    /**
     * 批量删除设备模型
     *
     * @param deviceModeIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteHwDeviceModeByDeviceModeIds(Long[] deviceModeIds);

    /**
     * 批量删除设备模型功能
     *
     * @param deviceModeIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteHwDeviceModeFunctionByDeviceModeIds(Long[] deviceModeIds);

    /**
     * 批量新增设备模型功能
     *
     * @param hwDeviceModeFunctionList 设备模型功能列表
     * @return 结果
     */
    public int batchHwDeviceModeFunction(List<HwDeviceModeFunction> hwDeviceModeFunctionList);


    /**
     * 通过设备模型主键删除设备模型功能信息
     *
     * @param deviceModeId 设备模型ID
     * @return 结果
     */
    public int deleteHwDeviceModeFunctionByDeviceModeId(Long deviceModeId);

    /**
     * 新增设备模型功能
     *
     * @param hwDeviceModeFunction 设备模型功能
     * @return 结果
     */
    public int insertHwDeviceModeFunction(HwDeviceModeFunction hwDeviceModeFunction);

    /**
     * 修改设备模型功能
     *
     * @param hwDeviceModeFunction 设备模型功能
     * @return 结果
     */
    public int updateHwDeviceModeFunction(HwDeviceModeFunction hwDeviceModeFunction);

    /**
     * 删除设备模型功能
     *
     * @param modeFunctionId 设备模型功能主键
     * @return 结果
     */
    public int deleteHwDeviceModeFunctionByModeFunctionId(Long modeFunctionId);

    /**
     * 批量新增设备模型功能参数
     *
     * @param hwDeviceModeParameterList 设备模型功能参数列表
     * @return 结果
     */
    public int batchHwDeviceModeParameters(List<HwDeviceModeParameter> hwDeviceModeParameterList);

    /**
     * 通过设备模型功能主键删除设备模型功能参数信息
     *
     * @param modeFunctionId 设备模型功能ID
     * @return 结果
     */
    public int deleteHwDeviceModeParameterByModeFunctionId(Long modeFunctionId);

    /**
     * @param: modeFunctionId
     * @description 根据设备功能主键获取参数信息
     * @author xins
     * @date 2023-09-12 10:44
     * @return List<HwDeviceModeParameter>
     */
    public List<HwDeviceModeParameter> selectDeviceModeParametersByModeFunctionId(Long modeFunctionId);


    /**
     * 查询设备模型列表,Join租户和场景
     *
     * @param hwDeviceMode 设备模型
     * @return 设备模型集合
     */
    public List<HwDeviceMode> selectHwDeviceModeJoinList(HwDeviceMode hwDeviceMode);

    List<HwDeviceMode> selectModel(Long sceneId);
}
