package com.ruoyi.business.mapper;

import java.util.List;
import java.util.Map;

import com.alibaba.druid.support.http.MonitorClientContextListener;
import com.ruoyi.business.domain.HwMonitorUnit;
import com.ruoyi.business.domain.HwScene;
import com.ruoyi.business.domain.VO.*;
import org.apache.ibatis.annotations.Param;

/**
 * 监控单元Mapper接口
 *
 * @author YINQ
 * @date 2023-08-31
 */
public interface HwMonitorUnitMapper
{
    /**
     * 查询监控单元
     *
     * @param monitorUnitId 监控单元主键
     * @return 监控单元
     */
    public HwMonitorUnit selectHwMonitorUnitByMonitorUnitId(Long monitorUnitId);
    /**
     * 查询区域数量站场景区总数百分比(左上角)
     * */
    public List<HwMonitorUnitVo> selectMonitorPercentage();
    /**
     * 	展示总的设备数量和总的监控单元数量（最低级的监控单元）
     * */
    public List<AllNumsVo> selectAllNums();
    public List<AllNumsVo> selectsum();
    //查询id与name的结果集
    List<HwMonitorUnitVo> selectSceneNameById();
    Integer selectSubSetNums(Long sceneId);

    /**
     * 分页获取最子集监控单元列表
     * */
    public List<HwMonitorUnit> selectLimitSubMonitorUnit(HwMonitorUnit hwMonitorUnit);
    Integer selectReleatedDeviceIdNums(Long deviceCode);
    Integer selectDeviceNums(Long sceneId);
    Integer selectSubDeviceSum(Long sceneId);

    /**
     * 监控单元报警统计
     * */
    public  List<AlarmStatsVo> selectAlarmStats(Long sceneId);
    /**
     * 获取最子级监控单元数量、设备数量和在线设备数量
     * */
    public List<SubDeviceSumVo> selectSubDeviceSum();
    /**
     * 按电子围栏定位显示其下设备数量，以及报警的设备数量。
     * */
    public List<ElectronicNumVo> selectElectronicNum(Long sceneId);
    //查询场景下电子围栏
    public List<Long> selectEleInScene(Long sceneId);
    public String selectEleType2(Long electronicFenceId);
    public String selectEleType3(Long sceneId);

    public Integer selectDeviceType2Nums(Long sceneId);

    /**
     * 查询监控单元列表
     *
     * @param hwMonitorUnit 监控单元
     * @return 监控单元集合
     */
    public List<HwMonitorUnit> selectHwMonitorUnitList(HwMonitorUnit hwMonitorUnit);
    /**
     * 树状获取监控单元列表
     * */
    //查询子集
    public List<HwMonitorUnit> selectHwMonitorUnitByParent(Long monitorUnitId);
    //查询最小单元的详细信息
    public List<HwMonitorUnit> selectHwmonitorUnitSub(Long monitorUnitId);
    //查询是否为最小单元
    public String selectHwMonitorUnitIsParent(Long monitorUnitId);

    public List<MonitorUnitNumVo> selectHaveEleDevice(Long sceneId);
    public List<MonitorUnitNumVo> selectMonitorUnitHaveEle(Long sceneId);
    /**
     * 新增监控单元
     *
     * @param hwMonitorUnit 监控单元
     * @return 结果
     */
    public int insertHwMonitorUnit(HwMonitorUnit hwMonitorUnit);

    /**
     * 修改监控单元
     *
     * @param hwMonitorUnit 监控单元
     * @return 结果
     */
    public int updateHwMonitorUnit(HwMonitorUnit hwMonitorUnit);

    /**
     * 删除监控单元
     *
     * @param monitorUnitId 监控单元主键
     * @return 结果
     */
    public int deleteHwMonitorUnitByMonitorUnitId(Long monitorUnitId);

    /**
     * 批量删除监控单元
     *
     * @param monitorUnitIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteHwMonitorUnitByMonitorUnitIds(Long[] monitorUnitIds);


    /**
     * 查询监控单元列表
     *
     * @param hwMonitorUnit 监控单元
     * @return 监控单元集合
     */
    public List<HwMonitorUnit> selectHwMonitorUnitJoinList(HwMonitorUnit hwMonitorUnit);

    /**
     * 查询监控单元Join monitorunittypename
     *
     * @param monitorUnitId 监控单元主键
     * @return 监控单元
     */
    public HwMonitorUnit selectHwMonitorUnitJoinByMonitorUnitId(Long monitorUnitId);


    /**
     * 是否存在子节点
     *
     * @param monitorUnitId 监控单元ID
     * @return 结果
     */
    public int hasChildByMonitorUnitId(Long monitorUnitId);

    /**
     * 查询部门是否存在用户
     *
     * @param monitorUnitId 监控单元ID
     * @return 结果
     */
    public int checkMonitorUnitExistDevice(Long monitorUnitId);

    /**
     * 删除部门管理信息
     *
     * @param monitorUnitId 监控单元ID
     * @return 结果
     */
    public int deleteMonitorUnitById(Long monitorUnitId);

    /**
     * 查询信标设备数量
     * */
     Integer selectBeaconDeviceNums(Long sceneId);

     /**
      * 根据场景查询信标设备
      * */
     Integer[] selectBeaconDeviceBySceneId(Long sceneId);

     /**
      * 根据设备id查询其对应的电子围栏信息
      * */
    HwMonitorUnit selectMonitorUnit(Long deviceId);

    /**
     * 查询监控单元信息并返回map
     * */
    List<TreeDeviceVo> selectMonitorUnitReturnMap();

    /**
     * 根据区域id查询监控单元
     * */
    List<HwMonitorUnit> getMonitorUnitByAreaId(@Param(value = "tenantId") Long tenantId, @Param(value = "areaId") Long areaId);

    List<HwMonitorUnit> selectMonitorUnitListHavAreaId(Long tenantId);

}
