package com.ruoyi.business.service.impl;

import com.ruoyi.business.domain.HwAlarmInfo;
import com.ruoyi.business.domain.HwAlarmRule;
import com.ruoyi.business.domain.HwAlarmRuleLink;
import com.ruoyi.business.mapper.HwAlarmInfoMapper;
import com.ruoyi.business.mapper.HwAlarmRuleMapper;
import com.ruoyi.business.service.IHwAlarmRuleService;
import com.ruoyi.common.core.constant.HwDictConstants;
import com.ruoyi.common.core.utils.DateUtils;
import com.ruoyi.common.core.utils.RegexUtils;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.common.datascope.annotation.DataScope;
import com.ruoyi.common.security.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * 报警规则Service业务层处理
 *
 * @author xins
 * @date 2023-11-03
 */
@Service
public class HwAlarmRuleServiceImpl implements IHwAlarmRuleService {
    @Autowired
    private HwAlarmRuleMapper hwAlarmRuleMapper;
    @Autowired
    private HwAlarmInfoMapper hwAlarmInfoMapper;

    /**
     * 查询报警规则
     *
     * @param alarmRuleId 报警规则主键
     * @return 报警规则
     */
    @Override
    public HwAlarmRule selectHwAlarmRuleByAlarmRuleId(Long alarmRuleId) {
        HwAlarmRule alarmRule = hwAlarmRuleMapper.selectHwAlarmRuleByAlarmRuleId(alarmRuleId);
        String triggerExpression = alarmRule.getTriggerExpression();
        String triggerCondition = "";
        if (triggerExpression.indexOf("and") > 0) {
            triggerCondition = HwDictConstants.DATA_MORE_THAN_A_AND_LESS_THAN_B;
        } else if (triggerExpression.indexOf("or") > 0) {
            triggerCondition = HwDictConstants.DATA_LESS_THAN_A_OR_MORE_THAN_B;
        } else if (triggerExpression.indexOf(">") > 0) {
            triggerCondition = HwDictConstants.DATA_MORE_THAN_A;
        } else if (triggerExpression.indexOf("<") > 0) {
            triggerCondition = HwDictConstants.DATA_LESS_THAN_B;
        } else if (triggerExpression.indexOf("=") > 0) {
            triggerCondition = HwDictConstants.DATA_EQUAL_A;
        }

        alarmRule.setTriggerCondition(triggerCondition);
        String ruleModeFunctionIdentifier = RegexUtils.findContentInBrace(triggerExpression);
        alarmRule.setRuleModeFunctionIdentifier(ruleModeFunctionIdentifier);
        triggerExpression = triggerExpression.replaceAll(ruleModeFunctionIdentifier, "");
        List<String> triggerDataList = RegexUtils.findNumberInText(triggerExpression);
        if (triggerDataList.size() == 1) {
            alarmRule.setTriggerDataA(Double.valueOf(triggerDataList.get(0)));
        } else if (triggerDataList.size() > 1) {
            alarmRule.setTriggerDataA(Double.valueOf(triggerDataList.get(0)));
            alarmRule.setTriggerDataB(Double.valueOf(triggerDataList.get(1)));
        }

        alarmRule.setAlarmInfoFlag(HwDictConstants.ALARM_INFO_FLAG_NO);
        if (alarmRule.getRuleType().equals(HwDictConstants.ALARM_RULE_RULE_TYPE_DEVICE)) {
            HwAlarmInfo queryAlarmInfo = new HwAlarmInfo();
            queryAlarmInfo.setAlarmInfoType(HwDictConstants.ALARM_INFO_TYPE_DEVICE);
            queryAlarmInfo.setAlarmReleatedId(alarmRule.getAlarmRuleId());
            List<HwAlarmInfo> alarmInfos = hwAlarmInfoMapper.selectHwAlarmInfoList(queryAlarmInfo);

            if (alarmInfos != null && !alarmInfos.isEmpty()) {
                alarmRule.setAlarmInfoFlag(HwDictConstants.ALARM_INFO_FLAG_YES);
            }
        }

        return alarmRule;
    }


    /**
     * 查询报警规则列表
     *
     * @param hwAlarmRule 报警规则
     * @return 报警规则
     */
    @Override
    @DataScope(tenantAlias = "har")
    public List<HwAlarmRule> selectHwAlarmRuleList(HwAlarmRule hwAlarmRule) {
        return hwAlarmRuleMapper.selectHwAlarmRuleList(hwAlarmRule);
    }

    /**
     * 查询报警规则列表
     *
     * @param hwAlarmRule 报警规则
     * @return 报警规则
     */
    @Override
    @DataScope(tenantAlias = "har")
    public List<HwAlarmRule> selectHwAlarmRuleJoinList(HwAlarmRule hwAlarmRule) {
        hwAlarmRule.setRuleType(HwDictConstants.ALARM_RULE_RULE_TYPE_DEVICE);
        return hwAlarmRuleMapper.selectHwAlarmRuleJoinList(hwAlarmRule);
    }

    /**
     * 新增报警规则
     *
     * @param hwAlarmRule 报警规则
     * @return 结果
     */
    @Transactional
    @Override
    public int insertHwAlarmRule(HwAlarmRule hwAlarmRule) {
        String triggerExpression = this.getTriggerExpression(hwAlarmRule);
        hwAlarmRule.setTriggerExpression(triggerExpression);
        hwAlarmRule.setAlarmPushFlag(HwDictConstants.ALARM_PUSH_FLAG_NO);
        hwAlarmRule.setCreateTime(DateUtils.getNowDate());
        hwAlarmRule.setTenantId(SecurityUtils.getTenantId());
        hwAlarmRule.setCreateBy(SecurityUtils.getUsername());
        int rows = hwAlarmRuleMapper.insertHwAlarmRule(hwAlarmRule);
        insertHwAlarmRuleLink(hwAlarmRule);
        return rows;
    }


    /**
     * 修改报警规则
     *
     * @param hwAlarmRule 报警规则
     * @return 结果
     */
    @Transactional
    @Override
    public int updateHwAlarmRule(HwAlarmRule hwAlarmRule) {
        hwAlarmRule.setUpdateTime(DateUtils.getNowDate());
        hwAlarmRule.setUpdateBy(SecurityUtils.getUsername());

        HwAlarmInfo queryAlarmInfo = new HwAlarmInfo();
        queryAlarmInfo.setAlarmInfoType(HwDictConstants.ALARM_INFO_TYPE_DEVICE);
        queryAlarmInfo.setAlarmReleatedId(hwAlarmRule.getAlarmRuleId());
        List<HwAlarmInfo> alarmInfos = hwAlarmInfoMapper.selectHwAlarmInfoList(queryAlarmInfo);
        if (alarmInfos == null || alarmInfos.isEmpty()) {//没有关联报警信息，才能修改表达式
            String triggerExpression = this.getTriggerExpression(hwAlarmRule);
            hwAlarmRule.setTriggerExpression(triggerExpression);
        } else {
            hwAlarmRule.setTriggerExpression(null);
        }

        hwAlarmRuleMapper.deleteHwAlarmRuleLinkByAlarmRuleId(hwAlarmRule.getAlarmRuleId());
        if (hwAlarmRule.getLinkFlag().equals(HwDictConstants.ALARM_RULE_LINK_FLAG_YES)) {
            insertHwAlarmRuleLink(hwAlarmRule);
        }
        return hwAlarmRuleMapper.updateHwAlarmRule(hwAlarmRule);
    }

    /**
     * 批量删除报警规则
     *
     * @param alarmRuleIds 需要删除的报警规则主键
     * @return 结果
     */
    @Transactional
    @Override
    public int deleteHwAlarmRuleByAlarmRuleIds(Long[] alarmRuleIds) {
//        hwAlarmRuleMapper.deleteHwAlarmRuleLinkByAlarmRuleIds(alarmRuleIds);
        return hwAlarmRuleMapper.deleteHwAlarmRuleByAlarmRuleIds(alarmRuleIds);//逻辑删除，更新状态为9
    }

    /**
     * 删除报警规则信息
     *
     * @param alarmRuleId 报警规则主键
     * @return 结果
     */
    @Transactional
    @Override
    public int deleteHwAlarmRuleByAlarmRuleId(Long alarmRuleId) {
        hwAlarmRuleMapper.deleteHwAlarmRuleLinkByAlarmRuleId(alarmRuleId);
        return hwAlarmRuleMapper.deleteHwAlarmRuleByAlarmRuleId(alarmRuleId);
    }

    /**
     * 新增报警规则联动信息信息
     *
     * @param hwAlarmRule 报警规则对象
     */
    public void insertHwAlarmRuleLink(HwAlarmRule hwAlarmRule) {
        List<HwAlarmRuleLink> hwAlarmRuleLinkList = hwAlarmRule.getHwAlarmRuleLinkList();
        Long alarmRuleId = hwAlarmRule.getAlarmRuleId();
        if (StringUtils.isNotNull(hwAlarmRuleLinkList)) {
            List<HwAlarmRuleLink> list = new ArrayList<HwAlarmRuleLink>();
            for (HwAlarmRuleLink hwAlarmRuleLink : hwAlarmRuleLinkList) {
                hwAlarmRuleLink.setAlarmRuleId(alarmRuleId);
                list.add(hwAlarmRuleLink);
            }
            if (list.size() > 0) {
                hwAlarmRuleMapper.batchHwAlarmRuleLink(list);
            }
        }
    }

    /**
     * @return String
     * @param: alarmRule
     * @description 获取报警规则触发表达式
     * @author xins
     * @date 2023-11-06 10:49
     */
    private String getTriggerExpression(HwAlarmRule alarmRule) {
        String ruleModeFunctionIdentifier = alarmRule.getRuleModeFunctionIdentifier();
        String triggerCondition = alarmRule.getTriggerCondition();
        Double triggerDataA = alarmRule.getTriggerDataA();
        Double triggerDataB = alarmRule.getTriggerDataB();

        String triggerExpression = HwDictConstants.triggerConditionMap.get(triggerCondition);
        if (triggerDataA != null) {
            triggerExpression = triggerExpression.replaceAll(HwDictConstants.TRIGGER_DATA_A, String.valueOf(triggerDataA));
        }

        if (triggerDataB != null) {
            triggerExpression = triggerExpression.replaceAll(HwDictConstants.TRIGGER_DATA_B, String.valueOf(triggerDataB));
        }

        triggerExpression = triggerExpression.replaceAll(HwDictConstants.TRIGGER_CONDITION_FUNCTION_IDENTIFIER, ruleModeFunctionIdentifier);

        return triggerExpression;

    }
}
