package com.ruoyi.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.parser.Feature;
import com.ruoyi.business.domain.*;
import com.ruoyi.business.domain.VO.*;
import com.ruoyi.business.mapper.*;
import com.ruoyi.business.service.IHwDeviceService;
import com.ruoyi.common.core.constant.Constants;
import com.ruoyi.common.core.constant.HwDictConstants;
import com.ruoyi.common.core.constant.SecurityConstants;
import com.ruoyi.common.core.constant.TdEngineConstants;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.enums.DataTypeEnums;
import com.ruoyi.common.core.enums.DeviceTypeEnums;
import com.ruoyi.common.core.exception.ServiceException;
import com.ruoyi.common.core.utils.DateUtils;
import com.ruoyi.common.core.utils.NumberUtils;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.common.datascope.annotation.DataScope;
import com.ruoyi.common.redis.service.RedisService;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.system.api.domain.SysUser;
import com.ruoyi.tdengine.api.RemoteTdEngineService;
import com.ruoyi.tdengine.api.domain.*;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.data.redis.RedisProperties;
import org.springframework.data.redis.cache.RedisCache;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import sun.reflect.generics.tree.Tree;

import javax.annotation.Resource;
import java.io.File;
import java.sql.Timestamp;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 设备信息Service业务层处理
 *
 * @author xins
 * @date 2023-08-24
 */
@Service
public class HwDeviceServiceImpl implements IHwDeviceService {
    @Autowired
    private HwDeviceMapper hwDeviceMapper;
    @Autowired
    private HwDeviceModeMapper hwDeviceModeMapper;
    @Autowired
    private HwSceneMapper hwSceneMapper;
    @Autowired
    private HwMonitorUnitMapper hwMonitorUnitMapper;
    @Autowired
    private HwDeviceModeFunctionMapper hwDevieModeFunctionMapper;
    @Resource
    private RemoteTdEngineService remoteTdEngineService;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private HwAlarmInfoMapper hwAlarmInfoMapper;
    @Autowired
    private HwElectronicFenceMapper hwElectronicFenceMapper;
    @Autowired
    private RedisService redisService;
    @Autowired
    private HwFenceAreaMapper hwFenceAreaMapper;
    /**
     * 查询设备信息
     *
     * @param deviceId 设备信息主键
     * @return 设备信息
     */
    @Override
    public HwDevice selectHwDeviceByDeviceId(Long deviceId) {
        return hwDeviceMapper.selectHwDeviceByDeviceId(deviceId);
    }

    /**
     * 查询设备信息,返回监控单元名称
     *
     * @param deviceId 设备信息主键
     * @return 设备信息
     */

    @Override
    public HwDevice selectHwDeviceJoinByDeviceId(Long deviceId) {
        HwDevice device = hwDeviceMapper.selectHwDeviceJoinByDeviceId(deviceId);
        Long deviceModeId = device.getDeviceModeId();
        if (deviceModeId != null) {
            HwDeviceMode mode = hwDeviceModeMapper.selectHwDeviceModeByDeviceModeId(deviceModeId);
            device.setGpsFlag(mode != null ? mode.getGpsFlag() : HwDictConstants.DEVICEMODE_GPS_FLAG_NO);
        }
        return device;
    }

    /**
     * 查询设备信息列表
     *
     * @param hwDevice 设备信息
     * @return 设备信息
     */
    @Override
    @DataScope(tenantAlias = "hd")
    public List<HwDevice> selectHwDeviceList(HwDevice hwDevice) {
        return hwDeviceMapper.selectHwDeviceList(hwDevice);
    }

    /**
     * 查询关联设备信息列表
     *
     * @param deviceId 设备信息主键
     * @return 设备信息
     */
    @Override
    public List<HwDevice> selectLinkDeviceList(Long deviceId) {
        List<HwDevice> devices = hwDeviceMapper.selectLinkedDevices(deviceId);
        return devices;
    }

    /**
     * 新增设备信息
     *
     * @param hwDevice 设备信息
     * @return 结果
     */
    @Transactional
    @Override
    public int insertHwDevice(HwDevice hwDevice) {
        HwDevice duplicateDevice = hwDeviceMapper.selectHwDeviceByDeviceCode(hwDevice.getDeviceCode());
        if (duplicateDevice != null) {
            throw new ServiceException("设备编号已经存在，请修改");
        }
        hwDevice.setCreateTime(DateUtils.getNowDate());
        String deviceType = hwDevice.getDeviceType();
        int deviceId = hwDeviceMapper.insertHwDevice(hwDevice);
        switch (deviceType) {
            case HwDictConstants.DEVICE_TYPE_GATEWAY_DEVICE:
                this.updateMqttAuth(hwDevice);
                break;
            case HwDictConstants.DEVICE_TYPE_DIRECT_CONNECT_DEVICE:
                this.updateMqttAuth(hwDevice);
                this.createTdTable(hwDevice);
                break;
            case HwDictConstants.DEVICE_TYPE_GATEWAY_SUB_EQUIPMENT:
                this.createTdTable(hwDevice);
                break;
            default:
                throw new ServiceException("Unknown device type: " + deviceType);
        }
        this.createTdDeviceStatusTable(hwDevice);
        return deviceId;
    }

    /**
     * @param: hwDevice
     * @description 更新redis中的设备信息
     * @author xins
     * @date 2023-09-19 10:06
     */
    public void updateMqttAuth(HwDevice hwDevice) {
        try {
            HwScene scene = hwSceneMapper.selectHwSceneBySceneId(hwDevice.getSceneId());
            String modeAccount = scene.getModeAccount();
            String modeKey = scene.getModeKey();
            if (StringUtils.isNotEmpty(modeAccount) && StringUtils.isNotEmpty(modeKey)) {
                String deviceInfoStr = redisTemplate.opsForValue().get(HwDictConstants.REDIS_KEY_DEVICE_INFO);
                JSONArray deviceInfoJsonArr = new JSONArray();
                if (StringUtils.isNotEmpty(deviceInfoStr)) {
                    deviceInfoJsonArr = JSON.parseArray(deviceInfoStr);
                }
                boolean redisUpdated = false;//是否更新已有的值
                for (Object deviceInfoObj : deviceInfoJsonArr) {
                    JSONObject deviceInfoJson = (JSONObject) deviceInfoObj;
                    if (deviceInfoJson.getString("deviceCode").equals(hwDevice.getDeviceCode())) {
                        deviceInfoJson.put("userName", modeAccount);
                        deviceInfoJson.put("password", modeKey);
                        redisUpdated = true;
                    }
                }

                if (!redisUpdated) {
                    JSONObject deviceInfoJson = new JSONObject();
                    deviceInfoJson.put("deviceCode", hwDevice.getDeviceCode());
                    deviceInfoJson.put("userName", modeAccount);
                    deviceInfoJson.put("password", modeKey);
                    deviceInfoJsonArr.add(deviceInfoJson);
                }

                redisTemplate.opsForValue().set(HwDictConstants.REDIS_KEY_DEVICE_INFO, deviceInfoJsonArr.toString());
            }
        } catch (Exception e) {
            throw new RuntimeException(e.getMessage());
        }
    }

    /**
     * @param: hwDevice
     * @description 在tdengine创建table
     * @author xins
     * @date 2023-09-19 10:07
     */
    public void createTdTable(HwDevice hwDevice) {
        TdTableVo tdTableVo = new TdTableVo();
        String databaseName = TdEngineConstants.getDatabaseName();
        String superTableName = TdEngineConstants.getSuperTableName(hwDevice.getDeviceModeId());
        String tableName = TdEngineConstants.getDeviceDataTableName(hwDevice.getDeviceId());

        List<TdField> tagsFields = getTdTagsFields(hwDevice);

        tdTableVo.setDatabaseName(databaseName);
        tdTableVo.setSuperTableName(superTableName);
        tdTableVo.setTableName(tableName);
        tdTableVo.setTagsFieldValues(tagsFields);

        R<?> tdReturnMsg = this.remoteTdEngineService.createTable(tdTableVo, SecurityConstants.INNER);
        if (tdReturnMsg.getCode() != Constants.SUCCESS) {//抛出异常，回滚事务
            throw new RuntimeException(tdReturnMsg.getMsg());
        }
    }

    public void createTdDeviceStatusTable(HwDevice hwDevice) {
        TdTableVo tdTableVo = new TdTableVo();
        tdTableVo.setDatabaseName(TdEngineConstants.PLATFORM_DB_NAME);
        tdTableVo.setSuperTableName(TdEngineConstants.DEFAULT_DEVICE_STATUS_SUPER_TABLE_NAME);
        tdTableVo.setTableName(TdEngineConstants.getDeviceStatusTableName(hwDevice.getDeviceId()));

        List<TdField> tagsFields = getTdTagsFields(hwDevice);

        TdField sceneIdTag = new TdField();
        sceneIdTag.setFieldName(TdEngineConstants.ST_TAG_SCENEID);
        sceneIdTag.setFieldValue(hwDevice.getSceneId());
        tagsFields.add(sceneIdTag);

        tdTableVo.setTagsFieldValues(tagsFields);

        R<?> tdReturnMsg = this.remoteTdEngineService.createTable(tdTableVo, SecurityConstants.INNER);
        if (tdReturnMsg.getCode() != Constants.SUCCESS) {//抛出异常，回滚事务
            throw new RuntimeException(tdReturnMsg.getMsg());
        }
    }

    private List<TdField> getTdTagsFields(HwDevice hwDevice) {
        List<TdField> tagsFields = new ArrayList<>();

        TdField deviceIdTag = new TdField();
        deviceIdTag.setFieldName(TdEngineConstants.ST_TAG_DEVICEID);
        deviceIdTag.setFieldValue(hwDevice.getDeviceId());
        tagsFields.add(deviceIdTag);

        TdField deviceCodeTag = new TdField();
        deviceCodeTag.setFieldName(TdEngineConstants.ST_TAG_DEVICECODE);
        deviceCodeTag.setFieldValue(hwDevice.getDeviceCode());
        deviceCodeTag.setDataTypeCode(DataTypeEnums.NCHAR.getDataCode());
        tagsFields.add(deviceCodeTag);

        TdField deviceNameTag = new TdField();
        deviceNameTag.setFieldName(TdEngineConstants.ST_TAG_DEVICENAME);
        deviceNameTag.setFieldValue(hwDevice.getDeviceName());
        deviceNameTag.setDataTypeCode(DataTypeEnums.NCHAR.getDataCode());
        tagsFields.add(deviceNameTag);

        TdField deviceModeIdTag = new TdField();
        deviceModeIdTag.setFieldName(TdEngineConstants.ST_TAG_DEVICEMODEID);
        deviceModeIdTag.setFieldValue(hwDevice.getDeviceModeId());
        tagsFields.add(deviceModeIdTag);

        TdField monitorUnitIdTag = new TdField();
        monitorUnitIdTag.setFieldName(TdEngineConstants.ST_TAG_MONITORUNITID);
        monitorUnitIdTag.setFieldValue(hwDevice.getMonitorUnitId());
        tagsFields.add(monitorUnitIdTag);

        return tagsFields;
    }


    /**
     * 查询3.	设备各类型占所有设备数量百分比饼状图（左下角）
     */
    @Override
    public List<DeviceModeVo> selectDeviceModeNameVo() {
        List<DeviceModeVo> list = hwDeviceMapper.selectDeviceModeNameVo();
        Integer value = 0;
        if (list.size() > 10) {
            for (int i = 9; i < list.size(); i++) {
                if (list.get(i).sum == null) {
                    list.get(i).sum = 0;
                }
                value = value + list.get(i).sum;
            }
        }
//        DeviceModeVo hwMonitorUnitTypeVo = new DeviceModeVo();
//        hwMonitorUnitTypeVo.setDeviceModeName("其他");
//        hwMonitorUnitTypeVo.setSum(value);
        Integer sums = 0;
        for (int i = 0; i < list.size(); i++) {
            if (list.get(i).sum == null) {
                list.get(i).sum = 0;
            }
            sums = sums + list.get(i).sum;
        }
        for (int i = 0; i < list.size(); i++) {
            list.get(i).setPercentage(NumberUtils.getPercentage(list.get(i).getSum(), sums));
        }


        return list;
//        if (list.size() > 10) {
//            list.add(10, hwMonitorUnitTypeVo);
//            return list.subList(0, 10);
//        } else {
//            return list;
//        }
    }


    /**
     * 修改设备信息
     *
     * @param hwDevice 设备信息
     * @return 结果
     */
    @Transactional
    @Override
    public int updateHwDevice(HwDevice hwDevice) {
        HwDevice dbDevice = hwDeviceMapper.selectHwDeviceByDeviceId(hwDevice.getDeviceId());
        if (dbDevice.getDeviceStatus().equals(HwDictConstants.DEVICE_STATUS_PUBLISH)) {
            throw new ServiceException("已发布状态不能修改");
        }
        hwDevice.setUpdateTime(DateUtils.getNowDate());
        int rows = hwDeviceMapper.updateHwDevice(hwDevice);
        this.updateTdEngine(hwDevice, dbDevice);
        return rows;
    }

    /**
     * 修改设备状态
     *
     * @param hwDevice 设备信息
     * @return 结果
     */
    @Override
    public int changeDeviceStatus(HwDevice hwDevice) {
        HwDevice dbDevice = hwDeviceMapper.selectHwDeviceByDeviceId(hwDevice.getDeviceId());
        if (dbDevice.getDeviceStatus().equals(HwDictConstants.DEVICE_STATUS_PUBLISH)) {
            throw new ServiceException("已发布状态不能修改");
        }
        hwDevice.setUpdateBy(SecurityUtils.getUsername());
        Date currentDate = new Date();
        hwDevice.setUpdateTime(currentDate);
        hwDevice.setPublishTime(currentDate);
        return hwDeviceMapper.updateHwDevice(hwDevice);
    }

    /**
     * 批量删除设备信息
     *
     * @param deviceIds 需要删除的设备信息主键
     * @return 结果
     */
    @Override
    public int deleteHwDeviceByDeviceIds(Long[] deviceIds) {
        return hwDeviceMapper.deleteHwDeviceByDeviceIds(deviceIds);
    }

    /**
     * 查询网关设备是否存在子设备
     *
     * @param releatedDeviceId 网关设备ID
     * @return 结果 true 存在 false 不存在
     */
    @Override
    public boolean checkExistSubDevice(Long releatedDeviceId) {
        int result = hwDeviceMapper.checkExistSubDevice(releatedDeviceId);
        return result > 0;
    }

    /**
     * 删除设备信息信息（逻辑更新状态）
     *
     * @param deviceId 设备信息主键
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int deleteHwDeviceByDeviceId(Long deviceId) {
        //逻辑删除，修改状态为9，修改设备编号为null，
        // 可以不drop子表，drop了也不会省出空间，而且子表的名称是用主键ID作为后缀的，不影响后续表的新建
//        String databaseName = TdEngineConstants.getDatabaseName();
//        String tableName = TdEngineConstants.getDeviceDataTableName(deviceId);
//        TdTableDropVo tdTableDropVo = new TdTableDropVo();
//        tdTableDropVo.setDatabaseName(databaseName);
//        tdTableDropVo.setTableName(tableName);
        int rows = hwDeviceMapper.deleteHwDeviceByDeviceId(deviceId);
//        R<?> tdReturnMsg = this.remoteTdEngineService.dropTable(tdTableDropVo,SecurityConstants.INNER);
//        if (tdReturnMsg.getCode() != Constants.SUCCESS) {//抛出异常，回滚事务
//            throw new RuntimeException(tdReturnMsg.getMsg());
//        }

        return rows;
    }


    /**
     * @return Map<String, List < HwDeviceVo>>
     * @description 设备监测页面，根据监控单元获取设备监控信息
     * @author xins
     * @date 2023-09-15 16:59
     */
    @Override
    public Map<String, List<HwDeviceVo>> getDevicesByMonitor(HwDevice queryHwDevice) {
        Map<String, List<HwDeviceVo>> devicesMap = new HashMap<String, List<HwDeviceVo>>();
        List<HwDeviceVo> controlDeviceVos = new ArrayList<HwDeviceVo>();
        List<HwDeviceVo> acquisitionDeviceVos = new ArrayList<HwDeviceVo>();
        String databaseName = TdEngineConstants.getDatabaseName();

        List<HwDevice> hwDevices = hwDeviceMapper.selectHwDeviceListByMonitor(queryHwDevice);
        hwDevices.forEach(hwDevice -> {
            Long deviceId = hwDevice.getDeviceId();
            HwDeviceModeFunction queryDeviceModeFunction = new HwDeviceModeFunction();
            queryDeviceModeFunction.setDeviceModeId(hwDevice.getDeviceModeId());
            List<HwDeviceModeFunction> deviceModeFunctions = hwDevieModeFunctionMapper.selectHwDeviceModeFunctionList(queryDeviceModeFunction);

            String tableName = TdEngineConstants.getDeviceDataTableName(deviceId);
            TdSelectDto tdSelectDto = new TdSelectDto();
            tdSelectDto.setDatabaseName(databaseName);
            tdSelectDto.setTableName(tableName);
            List<TdField> schemaFieldValues = new ArrayList<TdField>();
            List<HwDeviceModeFunction> modeFunctions = new ArrayList<>();

            deviceModeFunctions.forEach(mf -> {
                //是属性的，并且是可显示的
                if (mf.getFunctionMode().equals(HwDictConstants.FUNCTION_MODE_ATTRIBUTE)
                        && !mf.getDisplayFlag().equals(HwDictConstants.DEVICE_MODE_FUNCTION_DISPLAY_FLAG_NO)) {
                    TdField field = new TdField();
                    String functionIdentifierTransfer = TdEngineConstants.TDENGINE_KEY_TRANSFER_MAP.get(mf.getFunctionIdentifier());
                    String functionIdentifier = functionIdentifierTransfer == null ? mf.getFunctionIdentifier() : functionIdentifierTransfer;
                    field.setFieldName(functionIdentifier);
                    schemaFieldValues.add(field);
                } else if (mf.getFunctionMode().equals(HwDictConstants.FUNCTION_MODE_SERVICE)) {
                    modeFunctions.add(mf);
                }

            });
            tdSelectDto.setSchemaFieldValues(schemaFieldValues);
            //从tdengine获取此设备的最新数据
            List<Map<String, Object>> deviceLatestDataMapList =
                    (List<Map<String, Object>>) this.remoteTdEngineService.getLatestData(tdSelectDto, SecurityConstants.INNER).getData();


            //获取此设备模型的功能列表
            //获取可控制设备：根据功能是属性的并且属性的数据类型是tinyint类型的（bool类型开关就是用tinyint），并且是读写的
//            List<HwDeviceModeFunction> dds = deviceModeFunctions.stream()
//                    .filter(dmf -> dmf.getFunctionMode().equals(HwDictConstants.FUNCTION_MODE_ATTRIBUTE)
//                            && dmf.getDataType() != null
//                            && String.valueOf(dmf.getDataType())
//                            .equals(String.valueOf(DataTypeEnums.TINYINT.getDataCode()))
//                            && dmf.getRwFlag()!=null
//                            && dmf.getRwFlag().equals(HwDictConstants.RW_FLAG_RW)
//                    ).collect(Collectors.toList());

            //转换成map，key为标识符，value为功能名称（主要tdengine存的是标识符，用来下面转换成功能名称）
            Map<String, String> functionIndentifierName = deviceModeFunctions.stream().collect(
                    Collectors.toMap(HwDeviceModeFunction::getFunctionIdentifier,
                            HwDeviceModeFunction::getFunctionName));
            Map<String, String> functionIndentifierNameMap = new HashMap<>();
            functionIndentifierName.forEach((key, val) ->
            {
                String keyTransfer = TdEngineConstants.TDENGINE_KEY_TRANSFER_MAP.get(key) == null ?
                        key : TdEngineConstants.TDENGINE_KEY_TRANSFER_MAP.get(key);
                functionIndentifierNameMap.put(keyTransfer, val);

            });
            //获取可控制设备，根据是否有服务判断
//            List<HwDeviceModeFunction> modeFunctions = deviceModeFunctions.stream()
//                    .filter(dmf -> dmf.getFunctionMode().equals(HwDictConstants.FUNCTION_MODE_SERVICE)
//                    ).collect(Collectors.toList());

            if (modeFunctions != null && modeFunctions.size() > 0) {
                modeFunctions.forEach(modeFunction -> {
                    HwDeviceVo hwDeviceVo = new HwDeviceVo();
                    hwDeviceVo.setDeviceId(deviceId);
                    hwDeviceVo.setDeviceType(HwDictConstants.CONTROL_DEVICE);
                    hwDeviceVo.setDeviceCode(hwDevice.getDeviceCode());
                    hwDeviceVo.setDeviceName(hwDevice.getDeviceName());
                    hwDeviceVo.setDevicePic(hwDevice.getDevicePic());
                    hwDeviceVo.setDeviceModeFunction(modeFunction);
                    controlDeviceVos.add(hwDeviceVo);
                });
            }

            //将functionIdetifier转换成功能名称
            Map<String, Object> ddValueMap = new HashMap<String, Object>();
            if (deviceLatestDataMapList != null) {
                deviceLatestDataMapList.forEach(deviceLatestDataMap -> {
                    Set<String> functionIdentifiers = deviceLatestDataMap.keySet();
                    for (String dataFunctionIdentifier : functionIdentifiers) {
                        if (functionIndentifierNameMap.get(dataFunctionIdentifier) != null) {
                            ddValueMap.put(functionIndentifierNameMap.get(dataFunctionIdentifier),
                                    deviceLatestDataMap.get(dataFunctionIdentifier));
                        }
                    }
                });
            }
            //不论是否是控制型设备都显示最新数据
            HwDeviceVo hwDeviceVo = new HwDeviceVo();
            hwDeviceVo.setDeviceId(deviceId);
            hwDeviceVo.setDeviceModeId(hwDevice.getDeviceModeId());
            hwDeviceVo.setDeviceCode(hwDevice.getDeviceCode());
            hwDeviceVo.setDeviceName(hwDevice.getDeviceName());
            hwDeviceVo.setDevicePic(hwDevice.getDevicePic());
            hwDeviceVo.setDeviceType(HwDictConstants.ACQUISITION_DEVICE);
            hwDeviceVo.setDeviceDataMap(ddValueMap);
            acquisitionDeviceVos.add(hwDeviceVo);

        });

        devicesMap.put(HwDictConstants.CONTROL_DEVICE_NAME, controlDeviceVos);
        devicesMap.put(HwDictConstants.ACQUISITION_DEVICE_NAME, acquisitionDeviceVos);

        return devicesMap;
    }

    /**
     * 查询设备信息列表,join监控单元、设备模型等表
     *
     * @param hwDevice 设备信息
     * @return 设备信息
     */
    @Override
    @DataScope(tenantAlias = "hd")
    public List<HwDevice> selectHwDeviceJoinList(HwDevice hwDevice) {
        return hwDeviceMapper.selectHwDeviceJoinList(hwDevice);
    }


    /**
     * 导入设备数据
     *
     * @param deviceImportVoList 设备数据列表
     * @param isUpdateSupport    是否更新支持，如果已存在，则进行更新数据
     * @param sceneId            场景ID
     * @return 结果
     */
    @Override
    public String importDevices(List<HwDeviceImportVo> deviceImportVoList, Boolean isUpdateSupport, Long sceneId) {
        if (StringUtils.isNull(deviceImportVoList) || deviceImportVoList.size() == 0) {
            throw new ServiceException("导入标准数据不能为空！");
        }
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();
        String operName = SecurityUtils.getUsername();
        Long tenantId = SecurityUtils.getTenantId();
        String infoSeparator = "-";

        //判断场景是否属于此租户
        HwScene scene = hwSceneMapper.selectHwSceneBySceneId(sceneId);
        if (!scene.getTenantId().equals(tenantId)) {
            throw new ServiceException("选择场景不属于此租户！");
        }

        for (HwDeviceImportVo deviceImportVo : deviceImportVoList) {
            try {
                String checkMsg = this.checkImportDeviceData(deviceImportVo, sceneId, infoSeparator);
                if (StringUtils.isNotEmpty(checkMsg)) {
                    failureNum++;
                    failureMsg.append(
                            "<br/>"
                                    + failureNum + "、"
                                    + checkMsg);

                } else {
                    String deviceCode = deviceImportVo.getDeviceCode();
                    String deviceName = deviceImportVo.getDeviceName();
                    String deviceType = deviceImportVo.getDeviceType();
                    String monitorUnitInfo = deviceImportVo.getMonitorUnitInfo();
                    String gateWayDeviceInfo = deviceImportVo.getReleatedDeviceInfo();
                    String deviceModeInfo = deviceImportVo.getDeviceModeInfo();

                    Long monitorUnitId = Long.valueOf(monitorUnitInfo.substring(0, monitorUnitInfo.indexOf(infoSeparator)));
                    Long gatewayDeviceId = null;
                    Long deviceModeId = null;

                    //根据设备类型名称获取code
                    String deviceTypeCode = DeviceTypeEnums.dataTypeCodeMap.get(deviceType);

                    //验证是否存在这个设备
                    HwDevice d = hwDeviceMapper.selectHwDeviceByDeviceCode(deviceImportVo.getDeviceCode());
                    if (d == null) {
                        HwDevice device = new HwDevice();
                        device.setDeviceCode(deviceCode);
                        device.setDeviceName(deviceName);
                        device.setDeviceType(deviceTypeCode);
                        device.setSceneId(sceneId);
                        device.setMonitorUnitId(monitorUnitId);

                        switch (deviceTypeCode) {
                            case HwDictConstants.DEVICE_TYPE_GATEWAY_DEVICE:
                                device.setNetworkingMode(HwDictConstants.DEVICE_NETWORKING_MODE_WIFI);
                                device.setAccessProtocol(HwDictConstants.DEVICE_ACCESS_PROTOCOL_MQTT);
                                device.setDataFormat(HwDictConstants.DEVICE_DATA_FORMAT_JSON);
                                break;
                            case HwDictConstants.DEVICE_TYPE_GATEWAY_SUB_EQUIPMENT:
                                gatewayDeviceId = Long.valueOf(gateWayDeviceInfo.substring(0, gateWayDeviceInfo.indexOf(infoSeparator)));
                                deviceModeId = Long.valueOf(deviceModeInfo.substring(0, deviceModeInfo.indexOf(infoSeparator)));
                                device.setDeviceModeId(deviceModeId);
                                device.setReleatedDeviceId(gatewayDeviceId);
                                break;
                            case HwDictConstants.DEVICE_TYPE_DIRECT_CONNECT_DEVICE:
                                deviceModeId = Long.valueOf(deviceModeInfo.substring(0, deviceModeInfo.indexOf(infoSeparator)));
                                device.setNetworkingMode(HwDictConstants.DEVICE_NETWORKING_MODE_WIFI);
                                device.setAccessProtocol(HwDictConstants.DEVICE_ACCESS_PROTOCOL_MQTT);
                                device.setDataFormat(HwDictConstants.DEVICE_DATA_FORMAT_JSON);
                                device.setDeviceModeId(deviceModeId);
                                break;
                        }

                        device.setTenantId(tenantId);
                        device.setCreateBy(operName);
                        device.setCreateTime(new Date());
                        hwDeviceMapper.insertHwDevice(device);
                        successNum++;
                        successMsg.append(
                                "<br/>" + successNum + "、 " + device.getDeviceName() + " 导入成功");
                    } else if (isUpdateSupport) {//不支持更新
//                    device.setDeviceId(d.getDeviceId());
//                    device.setUpdateBy(operName);
//                    device.setUpdateTime(new Date());
//                    hwDeviceMapper.updateHwDevice(device);
                        successNum++;
                        successMsg.append(
                                "<br/>" + successNum + "、" + deviceImportVo.getDeviceName() + " 更新成功");
                    } else {
                        failureNum++;
                        failureMsg.append(
                                "<br/>"
                                        + failureNum
                                        + "、"
                                        + deviceImportVo.getDeviceCode()
                                        + "、"
                                        + deviceImportVo.getDeviceName()
                                        + " 已存在");
                    }
                }
            } catch (Exception e) {
                failureNum++;
                String msg = "<br/>" + failureNum + "、设备名称 " + deviceImportVo.getDeviceName() + " 导入失败：";
                failureMsg.append(msg + e.getMessage());
            }
        }
        if (failureNum > 0) {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
            throw new ServiceException(failureMsg.toString());
        } else {
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
        }
        return successMsg.toString();
    }


    /**
     * @param: days
     * @description 计算前几天的设备在线数量
     * @author xins
     * @date 2023-09-20 19:28
     */
    @Override
    public void computeOnlineDevicecCount(int days) {
        JSONObject jsonObject = new JSONObject();
        for (int i = 1; i <= days; i++) {

            Date currentDate = new Date();
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(currentDate);
            calendar.add(Calendar.DAY_OF_MONTH, -i);
            calendar.set(Calendar.HOUR_OF_DAY, 0);
            calendar.set(Calendar.MINUTE, 0);
            calendar.set(Calendar.SECOND, 0);
            Long startTime = calendar.getTimeInMillis();//先是昨天 00:00
            calendar.set(Calendar.HOUR_OF_DAY, 23);
            calendar.set(Calendar.MINUTE, 59);
            calendar.set(Calendar.SECOND, 59);
            Long endTime = calendar.getTimeInMillis();//23:59.59
            Map<Long, Integer> onlineDeviceCountPerDay = computeDeviceCountPerDay(startTime, endTime);
            Set<Long> times = onlineDeviceCountPerDay.keySet();
            for (Long key : times) {
                jsonObject.put(String.valueOf(key), onlineDeviceCountPerDay.get(key));
            }
        }
        redisTemplate.opsForValue().set(HwDictConstants.REDIS_KEY_ONLINE_DEVICE_COUNT_INFO, jsonObject.toString());
    }


    /**
     * @param: deviceControlVo
     * @description 下发当网关，主题带设备编号
     * @author xins
     * @date 2023-09-25 14:56
     */
    @Override
    public void publishControlCommand(DeviceControlVo deviceControlVo) {
        JSONObject controlCommandJson = new JSONObject();
        StringBuilder controlCommandTopic = new StringBuilder();
        Long deviceId = deviceControlVo.getDeviceId();
        HwDevice device = hwDeviceMapper.selectHwDeviceByDeviceId(deviceId);
        String deviceType = device.getDeviceType();
        if (deviceType.equals(HwDictConstants.DEVICE_TYPE_GATEWAY_SUB_EQUIPMENT)) {
            Long gatewayDeviceId = device.getReleatedDeviceId();
            HwDevice gatewayDevice = hwDeviceMapper.selectHwDeviceByDeviceId(gatewayDeviceId);
            controlCommandTopic.append(StringUtils
                    .format(HwDictConstants.CONTROL_COMMAND_TOPIC_VALUE, gatewayDevice.getDeviceCode()));
        } else {
            controlCommandTopic.append(StringUtils
                    .format(HwDictConstants.CONTROL_COMMAND_TOPIC_VALUE, device.getDeviceCode()));
        }
        JSONObject payloadJson = new JSONObject();
        JSONObject paramJson = new JSONObject();
        paramJson.put(HwDictConstants.CONTROL_COMMAND_PAYLOAD_UID_KEY, device.getDeviceCode());//子设备或直连设备设备编号

        payloadJson.put(HwDictConstants.CONTROL_COMMAND_PAYLOAD_MID_KEY, "");//请求ID,后续增加数据库表的主键ID
        payloadJson.put(HwDictConstants.CONTROL_COMMAND_PAYLOAD_TIMESTAMP_KEY, System.currentTimeMillis() / 1000);//单位：s
        payloadJson.put(HwDictConstants.CONTROL_COMMAND_PAYLOAD_TYPE_KEY, deviceControlVo.getType());
        payloadJson.put(HwDictConstants.CONTROL_COMMAND_PAYLOAD_PARAM_KEY, paramJson);

        controlCommandJson.put(HwDictConstants.CONTROL_COMMAND_TOPIC_KEY, controlCommandTopic.toString());
        controlCommandJson.put(HwDictConstants.CONTROL_COMMAND_PAYLOAD_KEY, payloadJson.toString());

        redisTemplate.convertAndSend(HwDictConstants.CONTROL_COMMAND_REDIS_KEY, controlCommandJson.toString());
    }

    /**
     * @return HwDevice
     * @param: deviceCode
     * @description 根据设备编号获取设备信息
     * @author xins
     * @date 2023-09-28 13:24
     */
    @Override
    public HwDevice selectHwDeviceByDeviceCode(String deviceCode) {
        return hwDeviceMapper.selectHwDeviceByDeviceCode(deviceCode);
    }

    /**
     * @return String
     * @param: sceneId
     * @description 获取最近几天在线设备数量
     * @author xins
     * @date 2023-09-21 9:09
     */
    @Override
    public JSONObject getOnlineDevicesCount(Long sceneId) {
        JSONObject returnObj = new JSONObject();
        int onlineDevicesCount = hwDeviceMapper.getOnlineDeviceNum(sceneId);

        JSONObject sortedJsonObject = new JSONObject();

        String onlineDeviceCountJsonStr = redisTemplate.opsForValue().get(HwDictConstants.REDIS_KEY_ONLINE_DEVICE_COUNT_INFO);
        if (onlineDeviceCountJsonStr != null) {
            JSONObject jsonObject = JSONObject.parseObject(onlineDeviceCountJsonStr);

            jsonObject.forEach((key, value) -> {
                String dateStrKey = DateUtils.parseDateToStr(DateUtils.MM_DD, new Date(Long.parseLong(key)));
                sortedJsonObject.put(dateStrKey, value);
            });
        }

        returnObj.put("onlineDevicesTrend", sortedJsonObject);
        returnObj.put("onlineDevicesCount", onlineDevicesCount);
        return returnObj;
    }

    /**
     * @return DevicesInfoVo
     * @param: monitorUnitId
     * @description 根据监控单元ID获取设备情况（包括在线和离线设备数量，正常的设备数量和报警的设备数量）
     * @author xins
     * @date 2023-09-28 14:25
     */
    @Override
    public DevicesInfoVo getDevicesInfoByMonitorUnitId(Long monitorUnitId) {
        HwDevice queryDevice = new HwDevice();
        queryDevice.setMonitorUnitId(monitorUnitId);
        queryDevice.setDeviceStatus(HwDictConstants.DEVICE_STATUS_PUBLISH);
        List<HwDevice> devices = this.selectHwDeviceList(queryDevice);
        Map<String, List<HwDevice>> hwDevicesMap = devices.stream()
                .collect(Collectors.groupingBy(HwDevice::getDeviceType));
        List<HwDevice> gatewayDevices = hwDevicesMap.get(HwDictConstants.DEVICE_TYPE_GATEWAY_DEVICE);
        List<HwDevice> subDevices = hwDevicesMap.get(HwDictConstants.DEVICE_TYPE_GATEWAY_SUB_EQUIPMENT);
        List<HwDevice> directConnectDevices = hwDevicesMap.get(HwDictConstants.DEVICE_TYPE_DIRECT_CONNECT_DEVICE);

        int onlineDevicesCount = 0;
        int offlineDevicesCount = 0;
        //网关设备和网关子设备，根据网关设备状态来判断网关子设备的状态
        if (gatewayDevices != null && !gatewayDevices.isEmpty() && subDevices != null && !subDevices.isEmpty()) {
            //根据关联网关设备ID进行分组
            Map<Long, List<HwDevice>> releatedDeviceIdDevicesMap = subDevices.stream()
                    .collect(Collectors.groupingBy(HwDevice::getReleatedDeviceId));

            for (HwDevice gatewayDevice : gatewayDevices) {
                String onlineStatus = gatewayDevice.getOnlineStatus();
                //获取关联子设备的数量
                int gatewaySubPublishedDevicesCount =
                        releatedDeviceIdDevicesMap.get(gatewayDevice.getDeviceId()) == null ?
                                0 : releatedDeviceIdDevicesMap.get(gatewayDevice.getDeviceId()).size();
                if (onlineStatus.equals(HwDictConstants.DEVICE_ONLINE_STATUS_ONLINE)) {
                    onlineDevicesCount += gatewaySubPublishedDevicesCount;
                } else if (onlineStatus.equals(HwDictConstants.DEVICE_ONLINE_STATUS_OFFLINE)) {
                    offlineDevicesCount += gatewaySubPublishedDevicesCount;
                }
            }
        }

        //直连设备直接获取设备状态
        if (directConnectDevices != null && !directConnectDevices.isEmpty()) {
            for (HwDevice directConnectDevice : directConnectDevices) {
                String onlineStatus = directConnectDevice.getOnlineStatus();
                if (onlineStatus.equals(HwDictConstants.DEVICE_ONLINE_STATUS_ONLINE)) {
                    onlineDevicesCount += 1;
                } else if (onlineStatus.equals(HwDictConstants.DEVICE_ONLINE_STATUS_OFFLINE)) {
                    offlineDevicesCount += 1;
                }
            }
        }

        int alarmDevicesCount = hwAlarmInfoMapper.selectAlarmCountByMonitorUnitId(monitorUnitId);
        int normalDevicesCount = onlineDevicesCount + offlineDevicesCount - alarmDevicesCount;

        DevicesInfoVo devicesInfoVo = new DevicesInfoVo();
        devicesInfoVo.setOnlineDevicesCount(onlineDevicesCount);
        devicesInfoVo.setOfflineDevicesCount(offlineDevicesCount);
        devicesInfoVo.setNormalDevicesCount(normalDevicesCount);
        devicesInfoVo.setAlarmDevicesCount(alarmDevicesCount);

        return devicesInfoVo;
    }

    /**
     * @description 重建tdengine所有设备表(重新创建所有设备子表，对于已经创建的不再创建，只是创建没有在TDengine创建子表的设备进行重新创建)
     * @author xins
     * @date 2023-10-10 20:05
     */
    @Override
    public void rebuildTdTables() {
        HwDevice queryDevice = new HwDevice();
        queryDevice.setDeviceType(HwDictConstants.DEVICE_TYPE_GATEWAY_SUB_EQUIPMENT);
        List<HwDevice> devices = hwDeviceMapper.selectHwDeviceList(queryDevice);
        devices.forEach(device -> {
            this.createTdTable(device);
        });
    }

    public String getDeviceTslJson(Long deviceId) {
        HwDevice device = hwDeviceMapper.selectHwDeviceByDeviceId(deviceId);
        String deviceCode = device.getDeviceCode();
        String devideType = device.getDeviceType();

        return "";
    }


    /**
     * @return JSONObject
     * @param: device
     * @description 获取正上设备数量和有报警的设备数量
     * @author xins
     * @date 2023-10-20 10:08
     */
    @Override
    public JSONObject selectDeviceNum(HwDevice device) {
        JSONObject result = new JSONObject();
        int deviceNum = hwDeviceMapper.selectDeviceNumCount(device);
        int abnormalDeviceNum = hwDeviceMapper.selectAbnormalDeviceNumCount(device);
        result.put("normalDeviceNum", deviceNum - abnormalDeviceNum);
        result.put("abnormalDeviceNum", abnormalDeviceNum);
        return result;
    }


    /**
     * @param: hwDevice 前端传的对象
     * @param: dbDevice 目前数据库的对象
     * @description 更新TdEngine数据库tag
     * @author xins
     * @date 2023-09-19 10:55
     */
    public void updateTdEngine(HwDevice hwDevice, HwDevice dbDevice) {
        String deviceType = hwDevice.getDeviceType();
        String databaseName = TdEngineConstants.getDatabaseName();
        String tableName = TdEngineConstants.getDeviceDataTableName(hwDevice.getDeviceId());
        AlterTagVo alterTagVo = new AlterTagVo();
        alterTagVo.setDatabaseName(databaseName);
        alterTagVo.setTableName(tableName);
        R<?> tdReturnMsg;
        if (deviceType.equals(HwDictConstants.DEVICE_TYPE_DIRECT_CONNECT_DEVICE)
                || deviceType.equals(HwDictConstants.DEVICE_TYPE_GATEWAY_SUB_EQUIPMENT)) {
            if (!hwDevice.getDeviceCode().equals(dbDevice.getDeviceCode())) {
                alterTagVo.setTagName(TdEngineConstants.ST_TAG_DEVICECODE);
                alterTagVo.setTagValue("'" + hwDevice.getDeviceCode() + "'");

                tdReturnMsg = this.remoteTdEngineService.alterTableTag(alterTagVo, SecurityConstants.INNER);
                if (tdReturnMsg.getCode() != Constants.SUCCESS) {//抛出异常，回滚事务
                    throw new RuntimeException(tdReturnMsg.getMsg());
                }
            }
            if (!hwDevice.getDeviceName().equals(dbDevice.getDeviceName())) {
                alterTagVo.setTagName(TdEngineConstants.ST_TAG_DEVICENAME);
                alterTagVo.setTagValue("'" + hwDevice.getDeviceName() + "'");
                tdReturnMsg = this.remoteTdEngineService.alterTableTag(alterTagVo, SecurityConstants.INNER);
                if (tdReturnMsg.getCode() != Constants.SUCCESS) {//抛出异常，回滚事务
                    throw new RuntimeException(tdReturnMsg.getMsg());
                }
            }

            if (!hwDevice.getMonitorUnitId().equals(dbDevice.getMonitorUnitId())) {
                alterTagVo.setTagName(TdEngineConstants.ST_TAG_MONITORUNITID);
                alterTagVo.setTagValue(hwDevice.getMonitorUnitId());
                tdReturnMsg = this.remoteTdEngineService.alterTableTag(alterTagVo, SecurityConstants.INNER);
                if (tdReturnMsg.getCode() != Constants.SUCCESS) {//抛出异常，回滚事务
                    throw new RuntimeException(tdReturnMsg.getMsg());
                }
            }

            if (!hwDevice.getDeviceModeId().equals(dbDevice.getDeviceModeId())) {
                alterTagVo.setTagName(TdEngineConstants.ST_TAG_DEVICEMODEID);
                alterTagVo.setTagValue(hwDevice.getDeviceModeId());
                tdReturnMsg = this.remoteTdEngineService.alterTableTag(alterTagVo, SecurityConstants.INNER);
                if (tdReturnMsg.getCode() != Constants.SUCCESS) {//抛出异常，回滚事务
                    throw new RuntimeException(tdReturnMsg.getMsg());
                }
            }
        }
    }


    /**
     * 查询设备信息列表(只返回网关子设备和直连设备)
     *
     * @param hwDevice 设备信息
     * @return 设备信息
     */
    @Override
    @DataScope(tenantAlias = "hd")
    public List<HwDevice> selectHwDeviceListWithDeviceMode(HwDevice hwDevice) {
        return hwDeviceMapper.selectHwDeviceListByMonitor(hwDevice);
    }

    private Map<Long, Integer> computeDeviceCountPerDay(Long startTime, Long endTime) {
        //先增加需要查询的字段名称（由于tdengine用last和last_row查询时字段显示有bug，所以需要设置要查询的字段）
        List<TdField> schemaFields = new ArrayList<TdField>();
        TdField deviceTypeField = new TdField();
        deviceTypeField.setFieldName(TdEngineConstants.ST_TAG_DEVICETYPE);

        TdField onlineStatusField = new TdField();
        onlineStatusField.setFieldName(TdEngineConstants.ST_TAG_ONLINESTATUS);

        schemaFields.add(deviceTypeField);
        schemaFields.add(onlineStatusField);

        DeviceStatus queryGroupStatus = new DeviceStatus();
        queryGroupStatus.setStartTime(startTime);
        queryGroupStatus.setEndTime(endTime);
        queryGroupStatus.setSchemaFields(schemaFields);

        DeviceStatus queryLastStatus = new DeviceStatus();
        queryLastStatus.setEndTime(startTime);
        queryLastStatus.setSchemaFields(schemaFields);

//        System.out.println(DateUtils.dateTime(new Date(startTime)) + "----" + DateUtils.dateTime(new Date(endTime)));
        Map<Long, Integer> onlineDeviceCountByDay = new HashMap<Long, Integer>();

        //先获取starttime和endtime时间范围内在线的设备状态信息
        Map<Long, List<DeviceStatus>> groupDeviceStatuMapList =
                (Map<Long, List<DeviceStatus>>) this.remoteTdEngineService.getOnlineDevicesGroupByDay(queryGroupStatus, SecurityConstants.INNER).getData();
        if (groupDeviceStatuMapList == null) {
            groupDeviceStatuMapList = new HashMap<>();
        }

        //获取在endtime之前最后的设备状态信息（有些设备会一直在线，没有连续的上线下线记录，所以要获取最近的设备状态信息来补齐后面的设备在线情况）
        List<Map<String, Object>> lastDeviceStatuMapList =
                (List<Map<String, Object>>) this.remoteTdEngineService.getLastOnlineDevices(queryLastStatus, SecurityConstants.INNER).getData();
        if (lastDeviceStatuMapList == null || lastDeviceStatuMapList.isEmpty()) {
            onlineDeviceCountByDay.put(startTime, 0);//当天的在线设备数量为0
        } else {
            for (Map<String, Object> groupDeviceStatuMap : lastDeviceStatuMapList) {
                DeviceStatus ds = new DeviceStatus();
                for (Map.Entry<String, Object> entry : groupDeviceStatuMap.entrySet()) {
                    switch (entry.getKey()) {
                        case TdEngineConstants.ST_TAG_DEVICEID:
                            ds.setDeviceId((long) (Integer) entry.getValue());
                            break;
                        case TdEngineConstants.ST_TAG_DEVICETYPE:
                            ds.setDeviceType((Integer) entry.getValue());
                            break;
                        case TdEngineConstants.ST_TAG_ONLINESTATUS:
                            ds.setOnlineStatus((Integer) entry.getValue());
                            break;
                        default:
                            break;
                    }
                }

                List<DeviceStatus> dss = groupDeviceStatuMapList.get(ds.getDeviceId());//获取此设备是否有状态信息
                int count = 0;
                if (dss == null) {//如果没有获取则按照此设备最近的状态信息进行计算
                    if (String.valueOf(ds.getOnlineStatus())
                            .equals(HwDictConstants.DEVICE_ONLINE_STATUS_ONLINE)) {
                        count = this.getDeviceCount(ds);
                    }
                } else {//如果获取到就按最近的设备状态信息进行计算
                    count = this.getDeviceCount(dss.get(0));
                }

                if (onlineDeviceCountByDay.get(startTime) == null) {
                    onlineDeviceCountByDay.put(startTime, count);
                } else {
                    onlineDeviceCountByDay.put(startTime, onlineDeviceCountByDay.get(startTime) + count);
                }
            }
        }

        return onlineDeviceCountByDay;
    }

    private int getDeviceCount(DeviceStatus ds) {
        String deviceType = String.valueOf(ds.getDeviceType());
        if (deviceType.equals(HwDictConstants.DEVICE_TYPE_GATEWAY_DEVICE)) {
            HwDevice queryDevice = new HwDevice();
            queryDevice.setReleatedDeviceId(ds.getDeviceId());
            queryDevice.setDeviceStatus(HwDictConstants.DEVICE_STATUS_PUBLISH);
            List<HwDevice> deviceList = hwDeviceMapper.selectHwDeviceList(queryDevice);
            return deviceList == null ? 0 : deviceList.size();
        } else {
            return 1;
        }
    }


    /**
     * @return String
     * @param: deviceImportVo
     * @param: sceneId
     * @param: infoSeparator
     * @description 在导入设备信息时校验数据正确性
     * @author xins
     * @date 2023-11-10 10:50
     */
    private String checkImportDeviceData(HwDeviceImportVo deviceImportVo, Long sceneId, String infoSeparator) {
        String deviceCode = deviceImportVo.getDeviceCode();
        String deviceName = deviceImportVo.getDeviceName();
        String deviceType = deviceImportVo.getDeviceType();
        String monitorUnitInfo = deviceImportVo.getMonitorUnitInfo();
        String gateWayDeviceInfo = deviceImportVo.getReleatedDeviceInfo();
        String deviceModeInfo = deviceImportVo.getDeviceModeInfo();

        if (StringUtils.isEmpty(deviceCode) || StringUtils.isEmpty(deviceName)
                || StringUtils.isEmpty(deviceType) || StringUtils.isEmpty(monitorUnitInfo)) {
            return "设备编号、设备名称、监控单元和设备类型不能为空";
        }

        //判断是否是子级监控单元，并且属于此场景
        Long monitorUnitId = Long.valueOf(monitorUnitInfo.substring(0, monitorUnitInfo.indexOf(infoSeparator)));
        HwMonitorUnit monitorUnit = hwMonitorUnitMapper.selectHwMonitorUnitByMonitorUnitId(monitorUnitId);
        if (!monitorUnit.getSceneId().equals(sceneId)) {
            return "设备名称 " + deviceImportVo.getDeviceName() + " 监控单元不属于所选择场景";
        }
        if (monitorUnit.getParentId() != null && !monitorUnit.getParentId().equals(0L)) {
            return "设备名称 " + deviceImportVo.getDeviceName() + " 此监控单元不是子级监控单元";
        }


        //如果是直连设备或子设备，必选选择设备模型，判断设备模型是否存在，并且属于此场景
        if (deviceType.equals(DeviceTypeEnums.DIRECT_CONNECT_DEVICE.getDataType())
                || deviceType.equals(DeviceTypeEnums.SUB_DEVICE.getDataType())) {
            if (StringUtils.isEmpty(deviceModeInfo)) {
                return "设备名称 " + deviceImportVo.getDeviceName() + " 设备模型不能为空";
            }
            Long deviceModeId = Long.valueOf(deviceModeInfo.substring(0, deviceModeInfo.indexOf(infoSeparator)));
            HwDeviceMode deviceMode = hwDeviceModeMapper.selectHwDeviceModeByDeviceModeId(deviceModeId);
            if (deviceMode == null || deviceMode.getDeviceModeStatus().equals(HwDictConstants.DEVICE_MODE_STATUS_DELETED)) {
                return "设备名称 " + deviceImportVo.getDeviceName() + " 设备模型不存在";
            }

            if (!deviceMode.getSceneId().equals(sceneId)) {
                return "设备名称 " + deviceImportVo.getDeviceName() + " 设备模型不属于所选择场景";
            }
        }

        //如果是网关子设备，必须有关联网关设备,并且关联网关设备属于此场景
        if (deviceType.equals(DeviceTypeEnums.SUB_DEVICE.getDataType())) {
            if (StringUtils.isEmpty(gateWayDeviceInfo)) {
                return "设备名称 " + deviceImportVo.getDeviceName() + " 关联网关设备不能为空";
            }
            Long gatewayDeviceId = Long.valueOf(gateWayDeviceInfo.substring(0, gateWayDeviceInfo.indexOf(infoSeparator)));
            HwDevice gatewayDevice = hwDeviceMapper.selectHwDeviceByDeviceId(gatewayDeviceId);
            if (gatewayDevice == null || gatewayDevice.getDeviceStatus().equals(HwDictConstants.DEVICE_STATUS_DELETE)) {
                return "设备名称 " + deviceImportVo.getDeviceName() + " 关联网关设备不存在";
            }
            if (!gatewayDevice.getSceneId().equals(sceneId)) {
                return "设备名称 " + deviceImportVo.getDeviceName() + " 关联网关设备不属于所选择场景";
            }
        }

        return "";
    }


    /**
     * 根据条件分页查询离线报警规则已分配设备列表
     *
     * @param device 设备信息
     * @return 设备信息集合信息
     */
    @Override
    @DataScope(tenantAlias = "hd")
    public List<HwDevice> selectAllocatedList(HwDevice device) {
        return hwDeviceMapper.selectAllocatedList(device);
    }

    /**
     * 根据条件分页查询离线报警规则未分配设备列表
     *
     * @param device 设备信息
     * @return 设备信息集合信息
     */
    @Override
    @DataScope(tenantAlias = "hd")
    public List<HwDevice> selectUnallocatedList(HwDevice device) {
        return hwDeviceMapper.selectUnallocatedList(device);
    }


    /**
     *
     * 查询设备经纬度
     * */
    @Override
    public List<BeaconDevice> selectDeviceLatitudeAndLongitude(Long sceneId) {
        //根据场景查询下面的设备
        HwDevice device = new HwDevice();
        device.setSceneId(sceneId);
        List<HwDevice> hwDevices = hwDeviceMapper.selectHwDeviceList(device);
        Integer[] integers = hwMonitorUnitMapper.selectBeaconDeviceBySceneId(sceneId);
        //调用tde查询每个设备的经纬度
        if (integers.length!=0){
            TdSelectDto tdSelectDto = new TdSelectDto();
            tdSelectDto.setDatabaseName(TdEngineConstants.PLATFORM_DB_NAME);
            TdField tdField = new TdField();
            tdField.setFieldName("longitude");
            TdField tdField1 = new TdField();
            tdField1.setFieldName("latitude");
            List<TdField> fields = new ArrayList<TdField>();
            fields.add(tdField);
            fields.add(tdField1);
            tdSelectDto.setSchemaFieldValues(fields);
            List<BeaconDevice> lists = new ArrayList<BeaconDevice>();
            for (int i = 0;i<integers.length;i++){
                tdSelectDto.setTableName(TdEngineConstants.DEFAULT_TABLE_NAME_PREFIX+integers[i]);
                R<?> latestData = remoteTdEngineService.getLatestData(tdSelectDto, SecurityConstants.INNER);
                Object data = latestData.getData();
                String s = JSONObject.toJSONString(data);
                if (latestData.getCode()!=500){
                    String substring = s.substring(1, s.length() - 1);
                    if (!substring.isEmpty()){
                        Map map = JSONObject.parseObject(substring, Map.class);
                        if (map.size()==2){
                            BeaconDevice beaconDevice = new BeaconDevice();
                            beaconDevice.setDeviceId(hwDevices.get(i).getDeviceId());
                            beaconDevice.setDeviceName(hwDevices.get(i).getDeviceName());
                            beaconDevice.setLatitude(Double.valueOf( map.get("latitude").toString()));
                            beaconDevice.setLongitude(Double.valueOf( map.get("longitude").toString()));
                            beaconDevice.setRemark(hwDevices.get(i).getRemark());
                            beaconDevice.setDeviceLocation(hwDevices.get(i).getDeviceLocation());
                            beaconDevice.setDeviceLocation("白银市");
                            beaconDevice.setDeviceModeId(hwDevices.get(i).getDeviceModeId());
                            beaconDevice.setMonitorUnitName(hwDevices.get(i).getMonitorUnitName());
                            //查询是否报警
                            Long integer = hwDeviceMapper.selectDeviceIfAlarm(hwDevices.get(i).getDeviceId());
                            String ifAlarm = new String();
                            if (integer>0){
                                //正常
                                ifAlarm="1";
                            }
                            else {
                                ifAlarm="0";
                            }
                            beaconDevice.setIfAlarm(ifAlarm);
                            lists.add(beaconDevice);
                        }
                    }

                }

            }
            return lists;
        }
        return null;

    }

    /**
     * 根据设备id查询信标设备历史数据
     * */
    @Override
    public List<Map<String, Object>> selectBeaconDevicesHistory(Map map) {
        try{
            TdHistorySelectDto tdHistorySelectDto   = new TdHistorySelectDto();
            tdHistorySelectDto.setDatabaseName(TdEngineConstants.PLATFORM_DB_NAME);
            tdHistorySelectDto.setTableName(TdEngineConstants.DEFAULT_TABLE_NAME_PREFIX+map.get("deviceId"));
            tdHistorySelectDto.setFirstFieldName(TdEngineConstants.DEFAULT_FIRST_FIELD_NAME);
            if (map.get("pageSize")!=null){
                tdHistorySelectDto.setLimit(Integer.valueOf(map.get("pageSize").toString()));
            }
            if (map.get("pageSize")!=null&&map.get("pageNum")!=null){
                tdHistorySelectDto.setOffset(Integer.valueOf(map.get("pageSize").toString())*(Integer.valueOf(map.get("pageNum").toString())-1));
            }
            tdHistorySelectDto.setOrderByFieldName(TdEngineConstants.DEFAULT_FIRST_FIELD_NAME);
            tdHistorySelectDto.setSort(TdEngineConstants.DEFAULT_ORDER_BY_MODE);
            if (map.containsKey("startTime")&&map.get("startTime")!=null){
                tdHistorySelectDto.setStartTime(Timestamp.valueOf(map.get("startTime").toString()).getTime());
            }
            if (map.containsKey("endTime")&&map.get("endTime")!=null){
                tdHistorySelectDto.setEndTime(Timestamp.valueOf(map.get("endTime").toString()).getTime());
            }
            R<TdReturnDataVo> historyData = remoteTdEngineService.getHistoryData(tdHistorySelectDto, SecurityConstants.INNER);
            System.out.println("查询历史数据"+map.get("deviceId"));
            List<Map<String, Object>> dataList = historyData.getData().dataList;
            for (int i = 0; i < dataList.size(); i++){
                dataList.get(i).put("deviceId",map.get("deviceId"));
            }
            int count = historyData.getData().count;
            Map<String, Object> map1 = new HashMap<>();
            map1.put("total",count);
            dataList.add(map1);
            return dataList;
        }catch (Exception ex){
            return null;
        }
    }

    /**
     * 根据设备id查询其对应的监控单元
     * */
    @Override
    public HwMonitorUnit selectMonitorUnit(Long deviceId) {
        HwMonitorUnit hwMonitorUnit = hwMonitorUnitMapper.selectMonitorUnit(deviceId);
        return hwMonitorUnit;
    }

    /**
     * 历史数据弹出框（多参数且可多选）
     * */
    @Override
    public List<Map<String, Object>> selectHistoryDevice(Map map) {
        //1-趋势分析 2历史数据
        //如果是趋势分析就删除分页
        Long pageSize = 0l;
        if (map.get("type")!=null){
            if (map.get("type").toString().equals("1")){
                map.remove("pageSize");
                map.remove("pageNum");
            }
            else {
                pageSize = Long.valueOf(map.get("pageSize").toString());
            }
        }

        String deviceIds = map.get("deviceIds").toString();
        String[] str = deviceIds.split(",");
        map.replace("pageSize",pageSize/str.length);
        List<Map<String, Object>> list = new ArrayList<Map<String, Object>>();
        Long total = 0l;
        for (int i = 0; i < str.length; i++) {



            map.put("deviceId",str[i]);
            if (i== str.length-2){
                map.replace("pageSize",pageSize%str.length);
            }
            if (map.get("prop")!=null){
                //查询类型的字段名
                String s = hwDeviceMapper.selectPropBydeviceIdAndFunctionName(Long.valueOf(map.get("deviceId").toString())
                        , map.get("prop").toString());
                map.put("functionIdentifier",s);
            }
            List<Map<String, Object>> list1 = this.selectBeaconDevicesHistory(map);
            total = total+Long.valueOf(list1.get(list1.size()-1).get("total").toString());
            list1.remove(list1.size()-1);

            List<Map<String, Object>> list2 = new ArrayList<Map<String, Object>>();
            //在弹出框中进行对比时，对历史数据进行过滤，根据传入的传感器类型

            if (map.get("functionIdentifier")!=null) {
                for (int x = 0; x < list1.size(); x++) {
                    //查看数据中是否拥有所需要的数据，没有进入下一循环
                    if (list1.get(i).get(map.get("functionIdentifier").toString())==null){
                        continue;
                    }
                    Map<String, Object> map2 = new HashMap<String, Object>();
                    //获取出需要的数据存入返回值中
                    map2.put("deviceId", list1.get(x).get("deviceId"));
                    map2.put("ts", list1.get(x).get("ts"));
                    map2.put("deviceName", list1.get(x).get("deviceName"));
                    map2.put("functionName", map.get("prop"));
                    map2.put("functionIdentifier",
                            list1.get(x).get(map.get("functionIdentifier").toString()));
                    list.add(map2);
                }
            }
            else {
                list.addAll(list1);
            }
        }
        Map<String, Object> mapName = new HashMap<String, Object>();
        if (list.size()>0){
            Set<String> strings = list.get(0).keySet();
            List<String> list2 = new ArrayList<String>();
            list2.addAll(strings);
            for (int i = 0; i < list2.size(); i++) {
                //查询类型的字段名
                String s = hwDeviceMapper.selectFunctionNameByFunctionIdentifier(list2.get(i));
                mapName.put(list2.get(i),s);
            }
            list.add(mapName);
        }
        if (map.get("type")!=null&&map.get("type").equals("1") ){
        for (int i = 0; i < list.size(); i++) {
            if (list.get(i).containsKey("valid")){
                list.get(i).remove("valid");
            }
            if (list.get(i).containsKey("acc")){
                list.get(i).remove("acc");
            }
            if (list.get(i).containsKey("version")){
                list.get(i).remove("version");
            }
            if (list.get(i).containsKey("speed")){
                list.get(i).remove("speed");
            }
        }
        }

        Map<String, Object> mapTotal = new HashMap<String, Object>();
        mapTotal.put("total", total);
        list.add(mapTotal);
        return list;
    }

    /**
     * 返回一级、二级、传感器
     * */
    @Override
    public List<TreeDeviceVo> selectComparison() {
        List<TreeDeviceVo> sceneMap = hwSceneMapper.selectHwSceneListReturnMap();
        List<TreeDeviceVo> unitMap = hwMonitorUnitMapper.selectMonitorUnitReturnMap();
        List<TreeDeviceVo> deviceMap = hwDeviceMapper.selectDeviceList();
        for (int i = 0; i < unitMap.size(); i++) {
            List<TreeDeviceVo> list = new ArrayList<TreeDeviceVo>();
            for (int j = 0; j < deviceMap.size(); j++) {
                if (unitMap.get(i).getVoId() == deviceMap.get(j).getParentId()) {
                    list.add(deviceMap.get(j));
                }
            }
            unitMap.get(i).setVoList(list);
        }
        for (int i = 0; i < sceneMap.size(); i++) {
            List<TreeDeviceVo> list = new ArrayList<TreeDeviceVo>();
            for (int j = 0; j < unitMap.size(); j++) {
                if (sceneMap.get(i).getVoId().equals(unitMap.get(j).getParentId())) {
                    list.add(unitMap.get(j));
                }
            }
            sceneMap.get(i).setVoList(list);
        }

        return sceneMap;


    }

    /**
     * 导出excel
     * */
    @Override
    public List<Map<String, Object>> getExportDevice(Map map) {
        map.remove("pageSize");
        map.remove("pageNum");
        String deviceIds = map.get("deviceIds").toString();
        String[] str = deviceIds.split(",");
        List<Map<String, Object>> exportList = new ArrayList<Map<String, Object>>();
        for (int i = 0; i < str.length; i++) {
            map.put("deviceId",str[i]);

            List<Map<String, Object>> list1 = this.selectBeaconDevicesHistory(map);
            list1.remove(list1.size()-1);
            exportList.addAll(list1);

        }
        return exportList;
    }

    /**
     * 查询租户下的所有信标设备下挂到监控单元
     * */
    @Override
    public List<HwMonitorUnit> getDeviceByAreaId(Long tenantId) {
        //如果是管理员，则不对租户进行过滤，即可以看到城市下所有租户的设备信息
        if (tenantId==1l)
        {
            tenantId = null;
        }

        /**
         * 逻辑
         *先根据租户id查询所有的监控单元
         * 再根据监控单元查出所有信标设备然后返回
         * */
        HwMonitorUnit hwMonitorUnit = new HwMonitorUnit();
        hwMonitorUnit.setTenantId(tenantId);
        List<HwMonitorUnit> unitList = hwMonitorUnitMapper.selectMonitorUnitListHavAreaId(tenantId);
        for (int i = 0; i < unitList.size(); i++){
            List<HwDevice> deviceList = this.getDeviceListInMonitorUnit(unitList.get(i).getMonitorUnitId());
            unitList.get(i).setDevicesList(deviceList);
        }
//        List<TreeAreaVo> treeArea1 = hwDeviceMapper.getTreeArea(areaId);
//        for (int i=0;i<treeArea1.size();i++){
//            TreeAreaVo treeAreaVo = this.AreaRecursion(treeArea1.get(i),tenantId);
//            treeArea.add(treeAreaVo);
//        }
        return unitList;

    }

    /**
     * 查询电子围栏通过设备
     * */
    @Override
    public List<HwElectronicFence> selectMonitorElectronic(Long deviceId) {
        List<HwElectronicFence> hwElectronicFences = hwElectronicFenceMapper.selectMonitorElectronic(deviceId);
            for (int i = 0; i < hwElectronicFences.size();i++){
                HwFenceArea hwFenceArea = new HwFenceArea();
                hwFenceArea.setElectronicFenceId(hwElectronicFences.get(i).getElectronicFenceId());
                List<HwFenceArea> hwFenceAreaList = hwFenceAreaMapper.selectHwFenceAreaList(hwFenceArea);
                hwElectronicFences.get(i).setHwFenceAreaList(hwFenceAreaList);
            }
        return hwElectronicFences;
    }



    //查询监控单元下的信标设备
    public List<HwDevice> getDeviceListInMonitorUnit(Long monitorUnitId){
        List<HwDevice> deviceListInMonitorUnit = hwDeviceMapper.getDeviceListInMonitorUnit(monitorUnitId);
        Map<String, String> map = new HashMap<String, String>();
        map.put("pageNum","1");
        map.put("pageSize","1");
        for (int i=0;i<deviceListInMonitorUnit.size();i++){

            map.put("deviceId",deviceListInMonitorUnit.get(i).getDeviceId().toString());
            //根据设备id查询该设备是否有报警信息

            Long integer = hwDeviceMapper.selectDeviceIfAlarm(Long.valueOf(map.get("deviceId")));
            String ifAlarm = new String();
            if (integer>0){
                //正常
                ifAlarm="1";
            }
            else {
                ifAlarm="0";
            }
            List<Map<String, Object>> list = this.selectBeaconDevicesHistory(map);
//            list.remove(list.size()-1);
            System.out.println(list);
            for(int j=0;j<list.size();j++){
                        deviceListInMonitorUnit.get(i).setIfAlarm(ifAlarm);
//                System.out.println(list.get(j).get("longitude"));
                    if (list.get(j).containsKey("longitude")){
                        System.out.println(deviceListInMonitorUnit.get(j).getDeviceId());
                        deviceListInMonitorUnit.get(i).setLongitude(Double.valueOf(list.get(j).get("longitude").toString()));
                    }
                    if (list.get(j).containsKey("latitude")){
                        deviceListInMonitorUnit.get(i).setLatitude(Double.valueOf(list.get(j).get("latitude").toString()));
                    }


            }
        }



                return deviceListInMonitorUnit;
    }
    /**
     * 导入excel
     * */
    @Override
    public void insertRedis(JSONArray jsonArray,Long tenantId){
//        redisTemplate.opsForValue().set("test",jsonArray.toJSONString());
        if (redisService.hasKey("file"+tenantId)) {
            redisService.deleteObject("file"+tenantId);
        }
        redisService.setCacheObject("file"+tenantId,jsonArray.toJSONString(),99999l, TimeUnit.DAYS);
    }

    /**
     * 查看excel
     * */
    @Override
    public JSONObject selectRedisFile(Long tenantId){
        Object o = redisService.getCacheObject("file" + tenantId);
        JSONObject array = new JSONObject(true);
        array.put("content",o.toString());

        String s = o.toString();
        String[] split = s.split("},\\{");
        String s2 = split[0].substring(1) + "}";
        Map<String, Object> map = JSON.parseObject(s2,LinkedHashMap.class, Feature.OrderedField);
        Map<Long, String> linkedMap = new LinkedHashMap<Long, String>();
        Long along = 1l;
        for (String key : map.keySet()){
            linkedMap.put(along, key);
            along++;
        }
        array.put("Header",linkedMap);
        return array;
    }

    /**
     * 根据名字搜索信标设备
     * */
    @Override
    public List<HwDevice> selectMonitorUnitAndDeviceByName(Map map) {
        String name = map.get("name").toString();
        HwMonitorUnit hwMonitorUnit= new HwMonitorUnit();
//        System.out.println(name);
        hwMonitorUnit.setMonitorUnitName(name);
        List<HwMonitorUnit> monitorUnits = hwMonitorUnitMapper.selectHwMonitorUnitList(hwMonitorUnit);
        HwDevice hwDevice = new HwDevice();
        hwDevice.setDeviceName(name);
        List<HwDevice> deviceList = hwDeviceMapper.selectHwDeviceList(hwDevice);
        List<Long> stringList = new ArrayList<Long>();
        for (int i = 0; i < deviceList.size(); i++){
            HwMonitorUnit hwMonitorUnit1 = hwMonitorUnitMapper.selectMonitorUnit(deviceList.get(i).getDeviceId());
            monitorUnits.add(hwMonitorUnit1);
        }
        List<HwMonitorUnit> collect = monitorUnits.stream().collect(Collectors.collectingAndThen(
                Collectors.toCollection(()->new TreeSet<>(Comparator.comparing(HwMonitorUnit::getMonitorUnitId))
        ),ArrayList::new));
        List<HwDevice> deviceListReturn = new ArrayList<HwDevice>();
        for (int i = 0; i < collect.size(); i++){
            deviceListReturn.addAll(this.getDeviceListInMonitorUnit(collect.get(i).getMonitorUnitId()));

        }
        return deviceListReturn;
    }

}
