package com.ruoyi.business.service.impl;

import java.util.List;

import com.ruoyi.business.domain.HwMonitorUnit;
import com.ruoyi.business.domain.VO.HwMonitorUnitVo;
import com.ruoyi.business.mapper.HwMonitorUnitMapper;
import com.ruoyi.common.core.exception.ServiceException;
import com.ruoyi.common.core.utils.DateUtils;
import com.ruoyi.common.datascope.annotation.DataScope;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.business.mapper.HwMonitorUnitTypeMapper;
import com.ruoyi.business.domain.HwMonitorUnitType;
import com.ruoyi.business.service.IHwMonitorUnitTypeService;

/**
 * 监控单元类型Service业务层处理
 *
 * @author YINQ
 * @date 2023-08-31
 */
@Service
public class HwMonitorUnitTypeServiceImpl implements IHwMonitorUnitTypeService {
    @Autowired
    private HwMonitorUnitTypeMapper hwMonitorUnitTypeMapper;

    @Autowired
    private HwMonitorUnitMapper hwMonitorUnitMapper;

    /**
     * 查询监控单元类型
     *
     * @param monitorUnitTypeId 监控单元类型主键
     * @return 监控单元类型
     */
    @Override
    public HwMonitorUnitType selectHwMonitorUnitTypeByMonitorUnitTypeId(Long monitorUnitTypeId) {
        return hwMonitorUnitTypeMapper.selectHwMonitorUnitTypeByMonitorUnitTypeId(monitorUnitTypeId);
    }

    /**
     * 查询监控单元类型列表
     *
     * @param hwMonitorUnitType 监控单元类型
     * @return 监控单元类型
     */
    @Override
    @DataScope(tenantAlias = "hmut")
    public List<HwMonitorUnitType> selectHwMonitorUnitTypeList(HwMonitorUnitType hwMonitorUnitType) {
        return hwMonitorUnitTypeMapper.selectHwMonitorUnitTypeList(hwMonitorUnitType);
    }


    /**
     * 新增监控单元类型
     *
     * @param hwMonitorUnitType 监控单元类型
     * @return 结果
     */
    @Override
    public int insertHwMonitorUnitType(HwMonitorUnitType hwMonitorUnitType) {
        hwMonitorUnitType.setCreateTime(DateUtils.getNowDate());
        return hwMonitorUnitTypeMapper.insertHwMonitorUnitType(hwMonitorUnitType);
    }

    /**
     * 修改监控单元类型
     *
     * @param hwMonitorUnitType 监控单元类型
     * @return 结果
     */
    @Override
    public int updateHwMonitorUnitType(HwMonitorUnitType hwMonitorUnitType) {
        hwMonitorUnitType.setUpdateTime(DateUtils.getNowDate());
        return hwMonitorUnitTypeMapper.updateHwMonitorUnitType(hwMonitorUnitType);
    }

    /**
     * 批量删除监控单元类型
     *
     * @param monitorUnitTypeIds 需要删除的监控单元类型主键
     * @return 结果
     */
    @Override
    public int deleteHwMonitorUnitTypeByMonitorUnitTypeIds(Long[] monitorUnitTypeIds) {
        for (Long monitorUnitTypeId : monitorUnitTypeIds) {
            HwMonitorUnit queryMonitorUnit = new HwMonitorUnit();
            queryMonitorUnit.setMonitorUnitTypeId(monitorUnitTypeId);
            List<HwMonitorUnit> monitorUnits = hwMonitorUnitMapper.selectHwMonitorUnitList(queryMonitorUnit);
            if (monitorUnits != null && monitorUnits.size() > 0) {
                throw new ServiceException("所选择删除的监控单元类型已关联监控单元，不能删除");
            }
        }
        return hwMonitorUnitTypeMapper.deleteHwMonitorUnitTypeByMonitorUnitTypeIds(monitorUnitTypeIds);
    }

    /**
     * 删除监控单元类型信息
     *
     * @param monitorUnitTypeId 监控单元类型主键
     * @return 结果
     */
    @Override
    public int deleteHwMonitorUnitTypeByMonitorUnitTypeId(Long monitorUnitTypeId) {
        return hwMonitorUnitTypeMapper.deleteHwMonitorUnitTypeByMonitorUnitTypeId(monitorUnitTypeId);
    }

    /**
     * 查询监控单元类型列表供编辑页面使用，不过滤权限，设置条件查询
     *
     * @param hwMonitorUnitType 监控单元类型
     * @return 监控单元类型集合
     */
    public List<HwMonitorUnitType> selectHwMonitorUnitTypeList4Select(HwMonitorUnitType hwMonitorUnitType) {
        return hwMonitorUnitTypeMapper.selectHwMonitorUnitTypeList(hwMonitorUnitType);
    }

    /**
     * 查询监控单元类型列表,Join租户和场景
     *
     * @param hwMonitorUnitType 监控单元类型
     * @return 监控单元类型
     */
    @Override
    @DataScope(tenantAlias = "hmut")
    public List<HwMonitorUnitType> selectHwMonitorUnitTypeJoinList(HwMonitorUnitType hwMonitorUnitType) {
        return hwMonitorUnitTypeMapper.selectHwMonitorUnitTypeJoinList(hwMonitorUnitType);
    }

}
